@@ -29,8 +29,11 @@
 
 #include "weechat.h"
 #include "completion.h"
+#include "alias.h"
 #include "command.h"
+#include "log.h"
 #include "utf8.h"
+#include "util.h"
 #include "weelist.h"
 #include "weeconfig.h"
 #include "../irc/irc.h"
@@ -53,13 +56,19 @@ completion_init (t_completion *completion, void *server, void *channel)
     completion->base_command = NULL;
     completion->base_command_arg = 0;
     completion->arg_is_nick = 0;
-    completion->position = -1;
     completion->base_word = NULL;
+    completion->base_word_pos = 0;
+    completion->position = -1; 
     completion->args = NULL;
     completion->direction = 0;
     
     completion->completion_list = NULL;
     completion->last_completion = NULL;
+
+    completion->word_found = NULL;
+    completion->position_replace = 0;
+    completion->diff_size = 0;
+    completion->diff_length = 0;
 }
 
 /*
@@ -109,7 +118,7 @@ completion_get_command_infos (t_completion *completion,
                               char **template, int *max_arg)
 {
     t_weechat_alias *ptr_alias;
-    char *ptr_command;
+    char *ptr_command, *ptr_command2, *pos;
     int i;
 #ifdef PLUGINS
     t_weechat_plugin *ptr_plugin;
@@ -128,6 +137,14 @@ completion_get_command_infos (t_completion *completion,
     }
     else
         ptr_command = completion->base_command;
+
+    ptr_command2 = strdup (ptr_command);
+    if (!ptr_command2)
+        return;
+    
+    pos = strchr (ptr_command2, ' ');
+    if (pos)
+        pos[0] = '\0';
     
 #ifdef PLUGINS
     /* look for plugin command handler */
@@ -139,10 +156,11 @@ completion_get_command_infos (t_completion *completion,
         {
             if ((ptr_handler->type == HANDLER_COMMAND)
                 && (ascii_strcasecmp (ptr_handler->command,
-                                      ptr_command) == 0))
+                                      ptr_command2) == 0))
             {
                 *template = ptr_handler->completion_template;
                 *max_arg = MAX_ARGS;
+                free (ptr_command2);
                 return;
             }
         }
@@ -153,10 +171,11 @@ completion_get_command_infos (t_completion *completion,
     for (i = 0; weechat_commands[i].command_name; i++)
     {
         if (ascii_strcasecmp (weechat_commands[i].command_name,
-                              ptr_command) == 0)
+                              ptr_command2) == 0)
         {
             *template = weechat_commands[i].completion_template;
             *max_arg = weechat_commands[i].max_arg;
+            free (ptr_command2);
             return;
         }
     }
@@ -166,14 +185,16 @@ completion_get_command_infos (t_completion *completion,
     {
         if ((irc_commands[i].cmd_function_args || irc_commands[i].cmd_function_1arg)
             && (ascii_strcasecmp (irc_commands[i].command_name,
-                                  ptr_command) == 0))
+                                  ptr_command2) == 0))
         {
             *template = irc_commands[i].completion_template;
             *max_arg = irc_commands[i].max_arg;
+            free (ptr_command2);
             return;
         }
     }
     
+    free (ptr_command2);
     return;
 }
 
@@ -340,6 +361,19 @@ completion_list_add_key_cmd (t_completion *completion)
     }
 }
 
+/*
+ * completion_list_add_self_nick: add self nick on server to completion list
+ */
+
+void
+completion_list_add_self_nick (t_completion *completion)
+{
+    if (completion->server)
+    {
+        completion_list_add (completion, ((t_irc_server *)(completion->server))->nick);
+    }
+}
+
 /*
  * completion_list_add_channel_nicks: add channel nicks to completion list
  */
@@ -747,6 +781,9 @@ completion_build_list_template (t_completion *completion, char *template)
                         case 'k': /* key cmd/funtcions*/
                             completion_list_add_key_cmd (completion);
                             break;
+                        case 'm': /* self nickname */
+                            completion_list_add_self_nick (completion);
+                            break;
                         case 'n': /* channel nicks */
                             completion_list_add_channel_nicks (completion);
                             break;
@@ -812,7 +849,8 @@ completion_build_list (t_completion *completion)
     int max_arg, i;
     
     completion_get_command_infos (completion, &template, &max_arg);
-    if (!template || (completion->base_command_arg > max_arg))
+    if (!template || (strcmp (template, "-") == 0) ||
+        (completion->base_command_arg > max_arg))
     {
         completion_stop (completion);
         return;
@@ -849,7 +887,7 @@ completion_find_context (t_completion *completion, char *buffer, int size, int p
     
     /* look for context */
     completion_free (completion);
-    command = (buffer[0] == '/') ? 1 : 0;
+    command = ((buffer[0] == '/') && (buffer[1] != '/')) ? 1 : 0;
     command_arg = 0;
     i = 0;
     while (i < pos)
@@ -1350,3 +1388,36 @@ completion_search (t_completion *completion, int direction,
         }
     }
 }
+
+/*
+ * completion_print_log: print completion list in log (usually for crash dump)
+ */
+
+void
+completion_print_log (t_completion *completion)
+{
+    weechat_log_printf ("[completion (addr:0x%X)]\n", completion);
+    weechat_log_printf ("  server . . . . . . . . : 0x%X\n", completion->server);
+    weechat_log_printf ("  channel. . . . . . . . : 0x%X\n", completion->channel);
+    weechat_log_printf ("  context. . . . . . . . : %d\n",   completion->context);
+    weechat_log_printf ("  base_command . . . . . : '%s'\n", completion->base_command);
+    weechat_log_printf ("  base_command_arg . . . : %d\n",   completion->base_command_arg);
+    weechat_log_printf ("  arg_is_nick. . . . . . : %d\n",   completion->arg_is_nick);
+    weechat_log_printf ("  base_word. . . . . . . : '%s'\n", completion->base_word);
+    weechat_log_printf ("  base_word_pos. . . . . : %d\n",   completion->base_word_pos);
+    weechat_log_printf ("  position . . . . . . . : %d\n",   completion->position);
+    weechat_log_printf ("  args . . . . . . . . . : '%s'\n", completion->args);
+    weechat_log_printf ("  direction. . . . . . . : %d\n",   completion->direction);
+    weechat_log_printf ("  completion_list. . . . : 0x%X\n", completion->completion_list);
+    weechat_log_printf ("  last_completion. . . . : 0x%X\n", completion->last_completion);
+    weechat_log_printf ("  word_found . . . . . . : '%s'\n", completion->word_found);
+    weechat_log_printf ("  position_replace . . . : %d\n",   completion->position_replace);
+    weechat_log_printf ("  diff_size. . . . . . . : %d\n",   completion->diff_size);
+    weechat_log_printf ("  diff_length. . . . . . : %d\n",   completion->diff_length);
+    if (completion->completion_list)
+    {
+        weechat_log_printf ("\n");
+        weelist_print_log (completion->completion_list,
+                           "completion list element");
+    }
+}
