@@ -34,6 +34,7 @@
 #include "weechat.h"
 #include "fifo.h"
 #include "command.h"
+#include "log.h"
 #include "weeconfig.h"
 #include "../gui/gui.h"
 
@@ -54,7 +55,7 @@ fifo_create ()
     
     if (cfg_irc_fifo_pipe)
     {
-        /* build FIFO filename: "~/.weechat/weechat_fifo_" + process PID */
+        /* build FIFO filename: "<weechat_home>/weechat_fifo_" + process PID */
         if (!weechat_fifo_filename)
         {
             filename_length = strlen (weechat_home) + 64;
@@ -101,20 +102,26 @@ fifo_exec (char *text)
     char *pos_msg, *pos;
     t_irc_server *ptr_server;
     t_irc_channel *ptr_channel;
-    t_gui_buffer *ptr_buffer;
     
     pos = NULL;
     ptr_server = NULL;
     ptr_channel = NULL;
-    ptr_buffer = NULL;
     
     /* look for server/channel at beginning of text */
     /* text may be: "server,channel *text" or "server *text" or "*text" */
     if (text[0] == '*')
     {
         pos_msg = text + 1;
-        ptr_buffer = (gui_current_window->buffer->has_input) ? gui_current_window->buffer : gui_buffers;
-        ptr_server = SERVER(ptr_buffer);
+        if (gui_current_window->buffer->has_input)
+        {
+            ptr_server = SERVER(gui_current_window->buffer);
+            ptr_channel = CHANNEL(gui_current_window->buffer);
+        }
+        else
+        {
+            ptr_server = SERVER(gui_buffers);
+            ptr_channel = NULL;
+        }
     }
     else
     {
@@ -150,7 +157,7 @@ fifo_exec (char *text)
             }
             if (ptr_server && pos)
             {
-                ptr_channel = channel_search (ptr_server, pos + 1);
+                ptr_channel = channel_search_any (ptr_server, pos + 1);
                 if (!ptr_channel)
                 {
                     irc_display_prefix (NULL, NULL, PREFIX_ERROR);
@@ -163,15 +170,7 @@ fifo_exec (char *text)
         }
     }
     
-    if (!ptr_buffer)
-    {
-        if (ptr_channel)
-            ptr_buffer = ptr_channel->buffer;
-        else
-            ptr_buffer = gui_buffers;
-    }
-    
-    user_command (ptr_buffer, ptr_server, pos_msg);
+    user_command (ptr_server, ptr_channel, pos_msg, 0);
 }
 
 /*
