@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 2003-2005 by FlashCode <flashcode@flashtux.org>
+ * Copyright (c) 2003-2006 by FlashCode <flashcode@flashtux.org>
  * See README for License detail, AUTHORS for developers list.
  *
  * This program is free software; you can redistribute it and/or modify
@@ -14,7 +14,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
 
@@ -30,8 +30,13 @@ struct t_history
     t_history *prev_history;    /* link to previous text/command            */
 };
 
-extern void history_add (void *, char *);
-extern void history_general_free ();
+extern t_history *history_global;
+extern t_history *history_global_last;
+extern t_history *history_global_ptr;
+
+extern void history_buffer_add (void *, char *);
+extern void history_global_add (char *);
+extern void history_global_free ();
 extern void history_buffer_free (void *);
 
 #endif /* history.h */
