@@ -36,6 +36,7 @@
 
 #include "weechat.h"
 #include "session.h"
+#include "hotlist.h"
 #include "log.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
@@ -176,6 +177,7 @@ session_save_channel (FILE *file, t_irc_channel *channel)
     rc = rc && (session_write_str (file, SESSION_CHAN_AWAY_MESSAGE, channel->away_message));
     rc = rc && (session_write_int (file, SESSION_CHAN_CYCLE, channel->cycle));
     rc = rc && (session_write_int (file, SESSION_CHAN_CLOSE, channel->close));
+    rc = rc && (session_write_int (file, SESSION_CHAN_DISPLAY_CREATION_DATE, channel->display_creation_date));
     rc = rc && (session_write_id  (file, SESSION_CHAN_END));
     
     if (!rc)
@@ -258,6 +260,7 @@ session_save_servers (FILE *file)
         rc = rc && (session_write_buf (file, SESSION_SERV_RECONNECT_START, &(ptr_server->reconnect_start), sizeof (time_t)));
         rc = rc && (session_write_int (file, SESSION_SERV_RECONNECT_JOIN, ptr_server->reconnect_join));
         rc = rc && (session_write_int (file, SESSION_SERV_IS_AWAY, ptr_server->is_away));
+        rc = rc && (session_write_str (file, SESSION_SERV_AWAY_MESSAGE, ptr_server->away_message));
         rc = rc && (session_write_buf (file, SESSION_SERV_AWAY_TIME, &(ptr_server->away_time), sizeof (time_t)));
         rc = rc && (session_write_int (file, SESSION_SERV_LAG, ptr_server->lag));
         rc = rc && (session_write_buf (file, SESSION_SERV_LAG_CHECK_TIME, &(ptr_server->lag_check_time), sizeof (struct timeval)));
@@ -292,8 +295,8 @@ session_save_dcc (FILE *file)
     
     rc = 1;
     
-    for (ptr_dcc = dcc_list; ptr_dcc;
-         ptr_dcc = ptr_dcc->next_dcc)
+    for (ptr_dcc = last_dcc; ptr_dcc;
+         ptr_dcc = ptr_dcc->prev_dcc)
     {
         rc = rc && (session_write_id  (file, SESSION_OBJ_DCC));
         rc = rc && (session_write_str (file, SESSION_DCC_SERVER, (ptr_dcc->server) ? ptr_dcc->server->name : NULL));
@@ -320,6 +323,9 @@ session_save_dcc (FILE *file)
         rc = rc && (session_write_buf (file, SESSION_DCC_LAST_ACTIVITY, &(ptr_dcc->last_activity), sizeof (time_t)));
         rc = rc && (session_write_buf (file, SESSION_DCC_BYTES_PER_SEC, &(ptr_dcc->bytes_per_sec), sizeof (unsigned long)));
         rc = rc && (session_write_buf (file, SESSION_DCC_ETA, &(ptr_dcc->eta), sizeof (unsigned long)));
+        rc = rc && (session_write_int (file, SESSION_DCC_CHILD_PID, ptr_dcc->child_pid));
+        rc = rc && (session_write_int (file, SESSION_DCC_CHILD_READ, ptr_dcc->child_read));
+        rc = rc && (session_write_int (file, SESSION_DCC_CHILD_WRITE, ptr_dcc->child_write));
         rc = rc && (session_write_id  (file, SESSION_DCC_END));
         
         if (!rc)
@@ -431,6 +437,33 @@ session_save_uptime (FILE *file)
     return rc;
 }
 
+/*
+ * session_save_hotlist: save hotlist into session file
+ */
+
+int
+session_save_hotlist (FILE *file)
+{
+    int rc;
+    t_weechat_hotlist *ptr_hotlist;
+    
+    rc = 1;
+    
+    for (ptr_hotlist = hotlist; ptr_hotlist;
+         ptr_hotlist = ptr_hotlist->next_hotlist)
+    {
+        rc = rc && (session_write_id  (file, SESSION_OBJ_HOTLIST));
+        rc = rc && (session_write_int (file, SESSION_HOTL_PRIORITY, ptr_hotlist->priority));
+        rc = rc && (session_write_str (file, SESSION_HOTL_SERVER, (ptr_hotlist->server) ? ptr_hotlist->server->name : NULL));
+        rc = rc && (session_write_int (file, SESSION_HOTL_BUFFER_NUMBER, ptr_hotlist->buffer->number));
+        rc = rc && (session_write_id  (file, SESSION_HOTL_END));
+        
+        if (!rc)
+            return 0;
+    }
+    return rc;
+}
+
 /*
  * session_save: save current session
  */
@@ -451,6 +484,7 @@ session_save (char *filename)
     rc = rc && (session_save_history (file, history_global_last));
     rc = rc && (session_save_buffers (file));
     rc = rc && (session_save_uptime (file));
+    rc = rc && (session_save_hotlist (file));
     
     fclose (file);
     
@@ -945,6 +979,9 @@ session_load_server (FILE *file)
             case SESSION_SERV_IS_AWAY:
                 rc = rc && (session_read_int (file, &(session_current_server->is_away)));
                 break;
+            case SESSION_SERV_AWAY_MESSAGE:
+                rc = rc && (session_read_str (file, &(session_current_server->away_message)));
+                break;
             case SESSION_SERV_AWAY_TIME:
                 rc = rc && (session_read_buf (file, &(session_current_server->away_time), sizeof (time_t)));
                 break;
@@ -968,7 +1005,8 @@ session_load_server (FILE *file)
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "server (object id: %d)\n"));
+                                      "server (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1065,9 +1103,13 @@ session_load_channel (FILE *file)
             case SESSION_CHAN_CLOSE:
                 rc = rc && (session_read_int (file, &(session_current_channel->close)));
                 break;
+            case SESSION_CHAN_DISPLAY_CREATION_DATE:
+                rc = rc && (session_read_int (file, &(session_current_channel->display_creation_date)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "channel (object id: %d)\n"));
+                                      "channel (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1137,7 +1179,8 @@ session_load_nick (FILE *file)
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "nick (object id: %d)\n"));
+                                      "nick (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1289,9 +1332,19 @@ session_load_dcc (FILE *file)
             case SESSION_DCC_ETA:
                 rc = rc && (session_read_buf (file, &(dcc->eta), sizeof (unsigned long)));
                 break;
+            case SESSION_DCC_CHILD_PID:
+                rc = rc && (session_read_int (file, &(dcc->child_pid)));
+                break;
+            case SESSION_DCC_CHILD_READ:
+                rc = rc && (session_read_int (file, &(dcc->child_read)));
+                break;
+            case SESSION_DCC_CHILD_WRITE:
+                rc = rc && (session_read_int (file, &(dcc->child_write)));
+                break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "DCC (object id: %d)\n"));
+                                      "DCC (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1341,7 +1394,8 @@ session_load_history (FILE *file)
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "history (object id: %d)\n"));
+                                      "history (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1443,7 +1497,8 @@ session_load_buffer (FILE *file)
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "buffer (object id: %d)\n"));
+                                      "buffer (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1520,7 +1575,8 @@ session_load_line (FILE *file)
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "line (object id: %d)\n"));
+                                      "line (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1557,7 +1613,67 @@ session_load_uptime (FILE *file)
                 break;
             default:
                 weechat_log_printf (_("session: warning: ignoring value from "
-                                      "uptime (object id: %d)\n"));
+                                      "uptime (object id: %d)\n"),
+                                    object_id);
+                rc = rc && (session_read_ignore_value (file));
+                break;
+        }
+    }
+    return 0;
+}
+
+/*
+ * session_load_hotlist: load hotlist from file
+ */
+
+int
+session_load_hotlist (FILE *file)
+{
+    int object_id, rc;
+    int priority;
+    char *server_name;
+    t_irc_server *ptr_server;
+    int buffer_number;
+    t_gui_buffer *ptr_buffer;
+
+    priority = 0;
+    ptr_server = NULL;
+    ptr_buffer = NULL;
+    
+    /* read hotlist values */
+    rc = 1;
+    while (rc)
+    {
+        if (feof (file))
+        {
+            session_crash (file, _("unexpected end of file (reading hotlist)"));
+            return 0;
+        }
+        if (fread ((void *)(&object_id), sizeof (int), 1, file) == 0)
+            return 0;
+        switch (object_id)
+        {
+            case SESSION_HOTL_END:
+                hotlist_add (priority, ptr_server, ptr_buffer, 1);
+                return 1;
+            case SESSION_HOTL_PRIORITY:
+                rc = rc && (session_read_int (file, &priority));
+                break;
+            case SESSION_HOTL_SERVER:
+                server_name = NULL;
+                if (!session_read_str (file, &server_name))
+                    return 0;
+                ptr_server = server_search (server_name);
+                free (server_name);
+                break;
+            case SESSION_HOTL_BUFFER_NUMBER:
+                rc = rc && (session_read_int (file, &buffer_number));
+                ptr_buffer = gui_buffer_search_by_number (buffer_number);
+                break;
+            default:
+                weechat_log_printf (_("session: warning: ignoring value from "
+                                      "history (object id: %d)\n"),
+                                    object_id);
                 rc = rc && (session_read_ignore_value (file));
                 break;
         }
@@ -1670,6 +1786,13 @@ session_load (char *filename)
                     return 0;
                 }
                 break;
+            case SESSION_OBJ_HOTLIST:
+                if (!session_load_hotlist (file))
+                {
+                    session_crash (file, _("failed to load hotlist"));
+                    return 0;
+                }
+                break;
             default:
                 weechat_log_printf (_("ignoring object (id: %d)\n"),
                                     object_id);
@@ -1700,7 +1823,7 @@ session_load (char *filename)
         irc_display_prefix (NULL, gui_current_window->buffer, PREFIX_ERROR);
         gui_printf_nolog (gui_current_window->buffer,
                           _("%s can't delete session file (%s)\n"),
-                          WEECHAT_ERROR);
+                          WEECHAT_ERROR, filename);
     }
     
     irc_display_prefix (NULL, gui_current_window->buffer, PREFIX_INFO);
