@@ -51,21 +51,19 @@
 #include <gnutls/gnutls.h>
 #endif
 
-#ifdef HAVE_ICONV
-#include <iconv.h>
-#endif
-
 #ifdef HAVE_LANGINFO_CODESET
 #include <langinfo.h>
 #endif
 
 #include "weechat.h"
+#include "alias.h"
 #include "backtrace.h"
 #include "command.h"
 #include "fifo.h"
 #include "log.h"
 #include "session.h"
 #include "utf8.h"
+#include "util.h"
 #include "weeconfig.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
@@ -93,238 +91,6 @@ gnutls_certificate_credentials gnutls_xcred;   /* gnutls client credentials */
 #endif
 
 
-/*
- * ascii_tolower: locale independant string conversion to lower case
- */
-
-void
-ascii_tolower (char *string)
-{
-    while (string && string[0])
-    {
-        if ((string[0] >= 'A') && (string[0] <= 'Z'))
-            string[0] += ('a' - 'A');
-        string++;
-    }
-}
-
-/*
- * ascii_toupper: locale independant string conversion to upper case
- */
-
-void
-ascii_toupper (char *string)
-{
-    while (string && string[0])
-    {
-        if ((string[0] >= 'a') && (string[0] <= 'z'))
-            string[0] -= ('a' - 'A');
-        string++;
-    }
-}
-
-/*
- * ascii_strcasecmp: locale and case independent string comparison
- */
-
-int
-ascii_strcasecmp (char *string1, char *string2)
-{
-    int c1, c2;
-    
-    if (!string1 || !string2)
-        return (string1) ? 1 : ((string2) ? -1 : 0);
-    
-    while (string1[0] && string2[0])
-    {
-        c1 = (int)((unsigned char) string1[0]);
-        c2 = (int)((unsigned char) string2[0]);
-        
-        if ((c1 >= 'A') && (c1 <= 'Z'))
-            c1 += ('a' - 'A');
-        
-        if ((c2 >= 'A') && (c2 <= 'Z'))
-            c2 += ('a' - 'A');
-        
-        if ((c1 - c2) != 0)
-            return c1 - c2;
-        
-        string1++;
-        string2++;
-    }
-    
-    return (string1[0]) ? 1 : ((string2[0]) ? -1 : 0);
-}
-
-/*
- * ascii_strncasecmp: locale and case independent string comparison
- *                    with max length
- */
-
-int
-ascii_strncasecmp (char *string1, char *string2, int max)
-{
-    int c1, c2, count;
-    
-    if (!string1 || !string2)
-        return (string1) ? 1 : ((string2) ? -1 : 0);
-    
-    count = 0;
-    while ((count < max) && string1[0] && string2[0])
-    {
-        c1 = (int)((unsigned char) string1[0]);
-        c2 = (int)((unsigned char) string2[0]);
-        
-        if ((c1 >= 'A') && (c1 <= 'Z'))
-            c1 += ('a' - 'A');
-        
-        if ((c2 >= 'A') && (c2 <= 'Z'))
-            c2 += ('a' - 'A');
-        
-        if ((c1 - c2) != 0)
-            return c1 - c2;
-        
-        string1++;
-        string2++;
-        count++;
-    }
-    
-    if (count >= max)
-        return 0;
-    else
-        return (string1[0]) ? 1 : ((string2[0]) ? -1 : 0);
-}
-
-/*
- * weechat_iconv: convert string to another charset
- */
-
-char *
-weechat_iconv (char *from_code, char *to_code, char *string)
-{
-    char *outbuf;
-    
-#ifdef HAVE_ICONV
-    iconv_t cd;
-    char *inbuf;
-    ICONV_CONST char *ptr_inbuf;
-    char *ptr_outbuf;
-    size_t inbytesleft, outbytesleft;
-    
-    if (from_code && from_code[0] && to_code && to_code[0]
-        && (ascii_strcasecmp(from_code, to_code) != 0))
-    {
-        cd = iconv_open (to_code, from_code);
-        if (cd == (iconv_t)(-1))
-            outbuf = strdup (string);
-        else
-        {
-            inbuf = strdup (string);
-            ptr_inbuf = inbuf;
-            inbytesleft = strlen (inbuf);
-            outbytesleft = inbytesleft * 4;
-            outbuf = (char *) malloc (outbytesleft + 2);
-            ptr_outbuf = outbuf;
-            iconv (cd, &ptr_inbuf, &inbytesleft, &ptr_outbuf, &outbytesleft);
-            if (inbytesleft != 0)
-            {
-                free (outbuf);
-                outbuf = strdup (string);
-            }
-            else
-                ptr_outbuf[0] = '\0';
-            free (inbuf);
-            iconv_close (cd);
-        }
-    }
-    else
-        outbuf = strdup (string);
-#else
-    /* make gcc happy */
-    (void) from_code;
-    (void) to_code;
-    outbuf = strdup (string);
-#endif /* HAVE_ICONV */
-    
-    return outbuf;
-}
-
-/*
- * weechat_strreplace: replace a string by new one in a string
- *                     note: returned value has to be free() after use
- */
-
-char *
-weechat_strreplace (char *string, char *search, char *replace)
-{
-    char *pos, *new_string;
-    int length1, length2, length_new, count;
-    
-    if (!string || !search || !replace)
-        return NULL;
-    
-    length1 = strlen (search);
-    length2 = strlen (replace);
-    
-    /* count number of strings to replace */
-    count = 0;
-    pos = string;
-    while (pos && pos[0] && (pos = strstr (pos, search)))
-    {
-        count++;
-        pos += length1;
-    }
-    
-    /* easy: no string to replace! */
-    if (count == 0)
-        return strdup (string);
-    
-    /* compute needed memory for new string */
-    length_new = strlen (string) - (count * length1) + (count * length2) + 1;
-    
-    /* allocate new string */
-    new_string = (char *)malloc (length_new * sizeof (char));
-    if (!new_string)
-        return strdup (string);
-    
-    /* replace all occurences */
-    new_string[0] = '\0';
-    while (string && string[0])
-    {
-        pos = strstr (string, search);
-        if (pos)
-        {
-            strncat (new_string, string, pos - string);
-            strcat (new_string, replace);
-            pos += length1;
-        }
-        else
-            strcat (new_string, string);
-        string = pos;
-    }
-    return new_string;
-}
-
-/*
- * get_timeval_diff: calculates difference between two times (return in milliseconds)
- */
-
-long
-get_timeval_diff (struct timeval *tv1, struct timeval *tv2)
-{
-    long diff_sec, diff_usec;
-    
-    diff_sec = tv2->tv_sec - tv1->tv_sec;
-    diff_usec = tv2->tv_usec - tv1->tv_usec;
-    
-    if (diff_usec < 0)
-    {
-        diff_usec += 1000000;
-        diff_sec--;
-    }
-    return ((diff_usec / 1000) + (diff_sec * 1000));
-}
-
 /*
  * weechat_display_usage: display WeeChat usage
  */
@@ -693,6 +459,7 @@ weechat_create_home_dirs ()
 {
     char *ptr_home;
     int dir_length;
+    struct stat statinfo;
 
     if (!weechat_home)
     {
@@ -715,6 +482,17 @@ weechat_create_home_dirs ()
         snprintf (weechat_home, dir_length, "%s%s.weechat", ptr_home,
                   DIR_SEPARATOR);
     }
+
+    /* if home already exists, it has to be a directory */
+    if (stat (weechat_home, &statinfo) == 0)
+    {
+        if (!S_ISDIR (statinfo.st_mode))
+        {
+            fprintf (stderr, _("%s home (%s) is not a directory\n"),
+                     WEECHAT_ERROR, weechat_home);
+            weechat_shutdown (EXIT_FAILURE, 0);
+        }
+    }
     
     /* create home directory; error is fatal */
     if (!weechat_create_dir (weechat_home))
@@ -900,7 +678,6 @@ weechat_dump (int crash)
     t_irc_server *ptr_server;
     t_irc_channel *ptr_channel;
     t_irc_nick *ptr_nick;
-    t_irc_dcc *ptr_dcc;
     t_gui_window *ptr_window;
     t_gui_buffer *ptr_buffer;
     
@@ -947,11 +724,9 @@ weechat_dump (int crash)
         }
     }
     
-    weechat_log_printf ("\n");
-    for (ptr_dcc = dcc_list; ptr_dcc; ptr_dcc = ptr_dcc->next_dcc)
-    {
-        dcc_print_log (ptr_dcc);
-    }
+    dcc_print_log ();
+    
+    gui_panel_print_log ();
     
     weechat_log_printf ("\n");
     weechat_log_printf ("[windows/buffers]\n");
@@ -978,7 +753,7 @@ weechat_dump (int crash)
         weechat_log_printf ("\n");
         gui_buffer_print_log (ptr_buffer);
     }
-
+    
     weechat_log_printf ("\n");
     ignore_print_log ();
     
@@ -1001,8 +776,6 @@ weechat_sigsegv ()
 
     fprintf (stderr, "\n");
     fprintf (stderr, "*** Very bad! WeeChat is crashing (SIGSEGV received)\n");
-    fprintf (stderr, "*** (%s, compiled on %s %s)\n",
-             PACKAGE_STRING, __DATE__, __TIME__);
     if (!weechat_log_crash_rename ())
         fprintf (stderr, "*** Full crash dump was saved to %s/weechat.log file.\n",
                  weechat_home);
@@ -1070,7 +843,8 @@ main (int argc, char *argv[])
     plugin_end ();                      /* end plugin interface(s)          */
 #endif
     server_disconnect_all ();           /* disconnect from all servers      */
-    (void) config_write (NULL);         /* save config file                 */
+    if (cfg_look_save_on_exit)
+        (void) config_write (NULL);     /* save config file                 */
     command_index_free ();              /* free commands index              */
     dcc_end ();                         /* remove all DCC                   */
     server_free_all ();                 /* free all servers                 */
