@@ -104,14 +104,7 @@ extern char *local_charset;
 extern gnutls_certificate_credentials gnutls_xcred;
 #endif
 
-extern void ascii_tolower (char *);
-extern void ascii_toupper (char *);
-extern int ascii_strcasecmp (char *, char *);
-extern int ascii_strncasecmp (char *, char *, int);
-extern char *weechat_iconv (char *, char *, char *);
-extern char *weechat_strreplace (char *, char *, char *);
 extern void weechat_dump (int);
-extern long get_timeval_diff (struct timeval *, struct timeval *);
 extern void weechat_shutdown (int, int);
 
 #endif /* weechat.h */
