@@ -34,7 +34,9 @@
 #include "../gui.h"
 #include "../../common/fifo.h"
 #include "../../common/utf8.h"
+#include "../../common/util.h"
 #include "../../common/weeconfig.h"
+#include "../../irc/irc.h"
 #include "gui-curses.h"
 
 #ifdef PLUGINS
@@ -42,6 +44,20 @@
 #endif
 
 
+int send_irc_quit = 0;
+
+
+/*
+ * gui_main_quit: quit weechat (signal received)
+ */
+
+void
+gui_main_quit ()
+{
+    quit_weechat = 1;
+    send_irc_quit = 1;
+}
+
 /*
  * gui_main_loop: main loop for WeeChat with ncurses GUI
  */
@@ -60,6 +76,7 @@ gui_main_loop ()
     struct tm *local_time;
     
     quit_weechat = 0;
+    send_irc_quit = 0;
 
     new_time = time (NULL);
     gui_last_activity_time = new_time;
@@ -69,11 +86,16 @@ gui_main_loop ()
     old_min = -1;
     old_sec = -1;
     check_away = 0;
+    
+    /* if SIGTERM or SIGHUP received => quit */
+    signal (SIGTERM, gui_main_quit);
+    signal (SIGHUP, gui_main_quit);
+    
     while (!quit_weechat)
     {
         /* refresh needed ? */
         if (gui_refresh_screen_needed)
-            gui_window_refresh_screen ();
+            gui_window_refresh_screen (0);
         
         new_time = time (NULL);
         local_time = localtime (&new_time);
@@ -112,7 +134,7 @@ gui_main_loop ()
             {
                 gui_infobar_draw_time (gui_current_window->buffer);
                 wmove (GUI_CURSES(gui_current_window)->win_input,
-                       0, gui_current_window->win_input_x);
+                       0, gui_current_window->win_input_cursor_x);
                 wrefresh (GUI_CURSES(gui_current_window)->win_input);
             }
             
@@ -260,6 +282,8 @@ gui_main_loop ()
         /* manages active DCC */
         dcc_handle ();
     }
+    if (send_irc_quit)
+        irc_cmd_send_quit (NULL, NULL, NULL);
 }
 
 /*
@@ -291,7 +315,7 @@ gui_main_init ()
     
     gui_infobar = NULL;
     
-    gui_ok = ((COLS > 5) && (LINES > 5));
+    gui_ok = ((COLS > WINDOW_MIN_WIDTH) && (LINES > WINDOW_MIN_HEIGHT));
 
     refresh ();
     
@@ -328,15 +352,14 @@ gui_main_end ()
     while (gui_panels)
         gui_panel_free (gui_panels);
 
-    /* delete all buffers */
-    gui_window_merge_all (gui_current_window);
-    while (gui_buffers)
-      gui_buffer_free (gui_buffers, 0);
-    
     /* delete all windows */
     while (gui_windows)
         gui_window_free (gui_windows);
     gui_window_tree_free (&gui_windows_tree);
+
+    /* delete all buffers */
+    while (gui_buffers)
+        gui_buffer_free (gui_buffers, 0);
     
     /* delete global history */
     history_global_free ();
