@@ -44,7 +44,7 @@ gui_status_draw (t_gui_buffer *buffer, int erase)
     t_gui_window *ptr_win;
     t_weechat_hotlist *ptr_hotlist;
     char format[32], str_nicks[32], *more;
-    int i, first_mode, x, server_pos, server_total;
+    int x, server_pos, server_total;
     int display_name, names_count;
     
     /* make gcc happy */
@@ -150,8 +150,7 @@ gui_status_draw (t_gui_buffer *buffer, int erase)
             wprintw (GUI_CURSES(ptr_win)->win_status, ":");
             gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_status,
                                           COLOR_WIN_STATUS_CHANNEL);
-            if (((CHANNEL(ptr_win->buffer)->type != CHANNEL_TYPE_PRIVATE)
-                 && (CHANNEL(ptr_win->buffer)->type != CHANNEL_TYPE_DCC_CHAT)
+            if (((CHANNEL(ptr_win->buffer)->type == CHANNEL_TYPE_CHANNEL)
                  && (!CHANNEL(ptr_win->buffer)->nicks))
                 || ((CHANNEL(ptr_win->buffer)->type == CHANNEL_TYPE_DCC_CHAT)
                     && (CHANNEL(ptr_win->buffer)->dcc_chat)
@@ -171,28 +170,10 @@ gui_status_draw (t_gui_buffer *buffer, int erase)
                     wprintw (GUI_CURSES(ptr_win)->win_status, "(");
                     gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_status,
                                                   COLOR_WIN_STATUS);
-                    i = 0;
-                    first_mode = 1;
-                    while (CHANNEL(ptr_win->buffer)->modes[i])
-                    {
-                        if (CHANNEL(ptr_win->buffer)->modes[i] != ' ')
-                        {
-                            if (first_mode)
-                            {
-                                wprintw (GUI_CURSES(ptr_win)->win_status, "+");
-                                first_mode = 0;
-                            }
-                            wprintw (GUI_CURSES(ptr_win)->win_status, "%c",
-                                     CHANNEL(ptr_win->buffer)->modes[i]);
-                        }
-                        i++;
-                    }
-                    if (CHANNEL(ptr_win->buffer)->modes[CHANNEL_MODE_KEY] != ' ')
-                        wprintw (GUI_CURSES(ptr_win)->win_status, ",%s",
-                                 CHANNEL(ptr_win->buffer)->key);
-                    if (CHANNEL(ptr_win->buffer)->modes[CHANNEL_MODE_LIMIT] != ' ')
-                        wprintw (GUI_CURSES(ptr_win)->win_status, ",%d",
-                                 CHANNEL(ptr_win->buffer)->limit);
+                    if (CHANNEL(ptr_win->buffer)->modes
+                        && (strcmp (CHANNEL(ptr_win->buffer)->modes, "+") != 0))
+                        wprintw (GUI_CURSES(ptr_win)->win_status, "%s",
+                                 CHANNEL(ptr_win->buffer)->modes);
                     gui_window_set_weechat_color (GUI_CURSES(ptr_win)->win_status,
                                                   COLOR_WIN_STATUS_DELIMITERS);
                     wprintw (GUI_CURSES(ptr_win)->win_status, ")");
@@ -361,10 +342,10 @@ gui_status_draw (t_gui_buffer *buffer, int erase)
         if (BUFFER_HAS_NICKLIST(ptr_win->buffer))
         {
             snprintf (str_nicks, sizeof (str_nicks) - 1, "%d", CHANNEL(ptr_win->buffer)->nicks_count);
-            x = ptr_win->win_width - strlen (str_nicks) - 4;
+            x = ptr_win->win_status_width - strlen (str_nicks) - 4;
         }
         else
-            x = ptr_win->win_width - 2;
+            x = ptr_win->win_status_width - 2;
         more = strdup (_("-MORE-"));
         x -= strlen (more) - 1;
         if (x < 0)
