@@ -41,6 +41,7 @@
 #include "../common/hotlist.h"
 #include "../common/log.h"
 #include "../common/utf8.h"
+#include "../common/util.h"
 #include "../irc/irc.h"
 
 
@@ -99,6 +100,7 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
             gui_buffers->all_servers = 1;
         if (cfg_log_auto_server)
             gui_log_start (gui_buffers);
+        gui_buffers->completion.server = server;
         return gui_buffers;
     }
     
@@ -237,8 +239,8 @@ gui_buffer_new (t_gui_window *window, void *server, void *channel, int type,
 t_gui_buffer *
 gui_buffer_search (char *server, char *channel)
 {
-    t_irc_server *ptr_server;
-    t_irc_channel *ptr_channel;
+    t_irc_server *ptr_server, *ptr_srv;
+    t_irc_channel *ptr_channel, *ptr_chan;
     t_gui_buffer *ptr_buffer;
     
     ptr_server = NULL;
@@ -256,12 +258,6 @@ gui_buffer_search (char *server, char *channel)
             if (!ptr_server)
                 return NULL;
         }
-        else
-        {
-            ptr_server = SERVER(gui_current_window->buffer);
-            if (!ptr_server)
-                ptr_server = SERVER(gui_buffers);
-        }
         
         if (channel && channel[0])
         {
@@ -271,6 +267,26 @@ gui_buffer_search (char *server, char *channel)
                 if (ptr_channel)
                     ptr_buffer = ptr_channel->buffer;
             }
+            else
+            {
+                for (ptr_srv = irc_servers; ptr_srv;
+                     ptr_srv = ptr_srv->next_server)
+                {
+                    for (ptr_chan = ptr_srv->channels; ptr_chan;
+                         ptr_chan = ptr_chan->next_channel)
+                    {
+                        if (ascii_strcasecmp (ptr_chan->name, channel) == 0)
+                        {
+                            ptr_channel = ptr_chan;
+                            break;
+                        }
+                    }
+                    if (ptr_channel)
+                        break;
+                }
+                if (ptr_channel)
+                    ptr_buffer = ptr_channel->buffer;
+            }
         }
         else
         {
@@ -288,6 +304,26 @@ gui_buffer_search (char *server, char *channel)
         gui_buffers : ptr_buffer;
 }
 
+/*
+ * gui_buffer_search_by_number: search a buffer by number
+ */
+
+t_gui_buffer *
+gui_buffer_search_by_number (int number)
+{
+    t_gui_buffer *ptr_buffer;
+    
+    for (ptr_buffer = gui_buffers; ptr_buffer;
+         ptr_buffer = ptr_buffer->next_buffer)
+    {
+        if (ptr_buffer->number == number)
+            return ptr_buffer;
+    }
+    
+    /* buffer not found */
+    return NULL;
+}
+
 /*
  * gui_buffer_find_window: find a window displaying buffer
  */
@@ -441,10 +477,6 @@ gui_buffer_free (t_gui_buffer *buffer, int switch_to_another)
     
     if (buffer->type == BUFFER_TYPE_RAW_DATA)
         gui_buffer_raw_data = NULL;
-
-    /* purge DCC chat using this buffer */
-    if (CHANNEL(buffer))
-        dcc_chat_remove_channel (CHANNEL(buffer));
     
     for (ptr_server = irc_servers; ptr_server;
          ptr_server = ptr_server->next_server)
@@ -573,7 +605,7 @@ gui_buffer_line_new (t_gui_buffer *buffer)
 void
 gui_buffer_merge_servers (t_gui_window *window)
 {
-    t_gui_buffer *ptr_buffer_server, *ptr_buffer;
+    t_gui_buffer *ptr_buffer_server, *ptr_buffer, *new_ptr_buffer;
     t_irc_server *ptr_server;
     
     /* new server buffer is the first server buffer found */
@@ -587,9 +619,9 @@ gui_buffer_merge_servers (t_gui_window *window)
     /* no server buffer found */
     if (!ptr_buffer_server)
         return;
-    
-    for (ptr_buffer = gui_buffers; ptr_buffer;
-         ptr_buffer = ptr_buffer->next_buffer)
+
+    ptr_buffer = gui_buffers;
+    while (ptr_buffer)
     {
         if ((ptr_buffer != ptr_buffer_server)
             && (BUFFER_IS_SERVER(ptr_buffer)))
@@ -617,12 +649,16 @@ gui_buffer_merge_servers (t_gui_window *window)
             
             /* free buffer but not lines, because they're now used by 
                our unique server buffer */
+            new_ptr_buffer = ptr_buffer->next_buffer;
             ptr_buffer->lines = NULL;
             gui_buffer_free (ptr_buffer, 1);
+            ptr_buffer = new_ptr_buffer;
             
             /* asociate server with new server buffer */
             ptr_server->buffer = ptr_buffer_server;
         }
+        else
+            ptr_buffer = ptr_buffer->next_buffer;
     }
     
     ptr_buffer_server->all_servers = 1;
@@ -910,6 +946,7 @@ gui_buffer_print_log (t_gui_buffer *buffer)
     weechat_log_printf ("  input_buffer_length. . : %d\n",   buffer->input_buffer_length);
     weechat_log_printf ("  input_buffer_pos . . . : %d\n",   buffer->input_buffer_pos);
     weechat_log_printf ("  input_buffer_1st_disp. : %d\n",   buffer->input_buffer_1st_display);
+    weechat_log_printf ("  completion . . . . . . : 0x%X\n", &(buffer->completion));
     weechat_log_printf ("  history. . . . . . . . : 0x%X\n", buffer->history);
     weechat_log_printf ("  last_history . . . . . : 0x%X\n", buffer->last_history);
     weechat_log_printf ("  ptr_history. . . . . . : 0x%X\n", buffer->ptr_history);
@@ -939,4 +976,7 @@ gui_buffer_print_log (t_gui_buffer *buffer)
         
         ptr_line = ptr_line->next_line;
     }
+    
+    weechat_log_printf ("\n");
+    completion_print_log (&(buffer->completion));
 }
