@@ -180,13 +180,13 @@ gui_add_to_line (t_gui_buffer *buffer, int type, char *nick, char *message)
                 buffer->notify_level)
             {
                 if (buffer->last_line->line_with_highlight)
-                    hotlist_add (HOTLIST_HIGHLIGHT, SERVER(buffer), buffer);
+                    hotlist_add (HOTLIST_HIGHLIGHT, SERVER(buffer), buffer, 0);
                 else if (BUFFER_IS_PRIVATE(buffer) && (buffer->last_line->line_with_message))
-                    hotlist_add (HOTLIST_PRIVATE, SERVER(buffer), buffer);
+                    hotlist_add (HOTLIST_PRIVATE, SERVER(buffer), buffer, 0);
                 else if (buffer->last_line->line_with_message)
-                    hotlist_add (HOTLIST_MSG, SERVER(buffer), buffer);
+                    hotlist_add (HOTLIST_MSG, SERVER(buffer), buffer, 0);
                 else
-                    hotlist_add (HOTLIST_LOW, SERVER(buffer), buffer);
+                    hotlist_add (HOTLIST_LOW, SERVER(buffer), buffer, 0);
                 gui_status_draw (gui_current_window->buffer, 1);
             }
         }
@@ -380,8 +380,8 @@ gui_printf_raw_data (void *server, int send, char *message)
                               GUI_COLOR(COLOR_WIN_CHAT_SERVER),
                               ((t_irc_server *)server)->name,
                               GUI_COLOR(COLOR_WIN_CHAT_DARK),
-                              GUI_COLOR(COLOR_WIN_CHAT_CHANNEL),
-                              (send) ? "<<--" : "-->>",
+                              GUI_COLOR((send) ? COLOR_WIN_CHAT_PART : COLOR_WIN_CHAT_JOIN),
+                              (send) ? PREFIX_PART : PREFIX_JOIN,
                               GUI_COLOR(COLOR_WIN_CHAT),
                               message);
             if (pos)
@@ -671,6 +671,7 @@ void
 gui_exec_action_dcc (t_gui_window *window, char *actions)
 {
     t_irc_dcc *dcc_selected, *ptr_dcc, *ptr_dcc_next;
+    t_gui_window *ptr_win;
     t_gui_buffer *ptr_buffer;
     
     while (actions[0])
@@ -705,7 +706,9 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
                 /* purge old DCC */
                 case 'p':
                 case 'P':
+                    window->dcc_first = NULL;
                     window->dcc_selected = NULL;
+                    window->dcc_last_displayed = NULL;
                     ptr_dcc = dcc_list;
                     while (ptr_dcc)
                     {
@@ -722,7 +725,13 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
                     if (gui_buffer_before_dcc)
                     {
                         ptr_buffer = window->buffer;
-                        gui_window_switch_to_buffer (window, gui_buffer_before_dcc);
+                        for (ptr_win = gui_windows; ptr_win;
+                             ptr_win = ptr_win->next_window)
+                        {
+                            if (ptr_win->buffer == ptr_buffer)
+                                gui_window_switch_to_buffer (ptr_win,
+                                                             gui_buffer_before_dcc);
+                        }
                         gui_buffer_free (ptr_buffer, 0);
                     }
                     else
@@ -759,6 +768,7 @@ gui_exec_action_dcc (t_gui_window *window, char *actions)
 void
 gui_exec_action_raw_data (t_gui_window *window, char *actions)
 {
+    t_gui_window *ptr_win;
     t_gui_buffer *ptr_buffer;
     
     while (actions[0])
@@ -768,13 +778,25 @@ gui_exec_action_raw_data (t_gui_window *window, char *actions)
             switch (actions[0])
             {
                 /* close raw IRC data */
+                case 'c':
+                case 'C':
+                    gui_buffer_clear (window->buffer);
+                    gui_window_redraw_buffer (window->buffer);
+                    return;
+                    break;
+                /* close raw IRC data */
                 case 'q':
                 case 'Q':
                     if (gui_buffer_before_raw_data)
                     {
                         ptr_buffer = window->buffer;
-                        gui_window_switch_to_buffer (window,
-                                                     gui_buffer_before_raw_data);
+                        for (ptr_win = gui_windows; ptr_win;
+                             ptr_win = ptr_win->next_window)
+                        {
+                            if (ptr_win->buffer == ptr_buffer)
+                                gui_window_switch_to_buffer (ptr_win,
+                                                             gui_buffer_before_raw_data);
+                        }
                         gui_buffer_free (ptr_buffer, 0);
                     }
                     else
