@@ -47,18 +47,18 @@ t_irc_command irc_commands[] =
     N_("[-all] [message]"),
     N_("   -all: toggle away status on all connected servers\n"
        "message: message for away (if no message is given, away status is removed)"),
-    "-all", 0, MAX_ARGS, 1, 1, NULL, irc_cmd_send_away, NULL },
+    "-all", 0, MAX_ARGS, 1, 0, NULL, irc_cmd_send_away, NULL },
   { "ban", N_("bans nicks or hosts"),
     N_("[channel] [nickname [nickname ...]]"),
     N_(" channel: channel for ban\n"
        "nickname: user or host to ban"),
     "%N", 0, MAX_ARGS, 0, 1, NULL, irc_cmd_send_ban, NULL },
   { "ctcp", N_("send a CTCP message (Client-To-Client Protocol)"),
-    N_("nickname type [arguments]"),
-    N_(" nickname: user to send CTCP to\n"
+    N_("receiver type [arguments]"),
+    N_(" receiver: nick or channel to send CTCP to\n"
        "     type: CTCP type (examples: \"version\", \"ping\", ..)\n"
        "arguments: arguments for CTCP"),
-    "%n action|ping|version", 2, MAX_ARGS, 1, 1, NULL, irc_cmd_send_ctcp, NULL },
+    "%c|%n action|ping|version", 2, MAX_ARGS, 1, 1, NULL, irc_cmd_send_ctcp, NULL },
   { "cycle", N_("leave and rejoin a channel"),
     N_("[channel[,channel]] [part_message]"),
     N_("     channel: channel name for cycle\n"
@@ -156,7 +156,7 @@ t_irc_command irc_commands[] =
        "  s: mark a user for receive server notices\n"
        "  w: user receives wallops\n"
        "  o: operator flag"),
-    "", 1, MAX_ARGS, 0, 1, NULL, irc_cmd_send_mode, irc_cmd_recv_mode },
+    "%c|%m", 1, MAX_ARGS, 0, 1, NULL, irc_cmd_send_mode, irc_cmd_recv_mode },
   { "motd", N_("get the \"Message Of The Day\""),
     N_("[target]"),
     N_("target: server name"),
@@ -258,7 +258,7 @@ t_irc_command irc_commands[] =
     N_("[channel] [topic]"),
     N_("channel: channel name\ntopic: new topic for channel "
        "(if topic is \"-delete\" then topic is deleted)"),
-    "%t", 0, MAX_ARGS, 1, 1, NULL, irc_cmd_send_topic, irc_cmd_recv_topic },
+    "%t|-delete", 0, MAX_ARGS, 1, 1, NULL, irc_cmd_send_topic, irc_cmd_recv_topic },
   { "trace", N_("find the route to specific server"),
     N_("[target]"),
     N_("target: server"),
@@ -288,7 +288,7 @@ t_irc_command irc_commands[] =
     "set the 'w' user mode for themselves"),
     N_("text"),
     N_("text to send"),
-    NULL, 1, MAX_ARGS, 1, 1, NULL, irc_cmd_send_wallops, NULL },
+    NULL, 1, MAX_ARGS, 1, 1, NULL, irc_cmd_send_wallops, irc_cmd_recv_wallops },
   { "who", N_("generate a query which returns a list of information"),
     N_("[mask [\"o\"]]"),
     N_("mask: only information which match this mask\n"
@@ -306,75 +306,9 @@ t_irc_command irc_commands[] =
        "  target: reply should match this mask"),
     "", 1, MAX_ARGS, 0, 1, NULL, irc_cmd_send_whowas, NULL },
   { "001", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "002", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "003", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "004", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_004 },
-  { "005", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "006", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "007", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "008", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "020", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "042", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "212", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "219", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_001 },
   { "221", N_("user mode string"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_221 },
-  { "250", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "251", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "252", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "253", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "254", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "255", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "256", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "257", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "258", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "259", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "260", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "261", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "262", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "263", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "264", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "265", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "266", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "267", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "268", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "269", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "290", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "292", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
   { "301", N_("away message"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_301 },
   { "302", N_("userhost"), "", "",
@@ -386,15 +320,15 @@ t_irc_command irc_commands[] =
   { "306", N_("now away"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_306 },
   { "307", N_("whois (registered nick)"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_307 },
-  { "310", N_("available for help"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_whois_nick_msg },
+  { "310", N_("whois (help mode)"), "", "",
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_310 },
   { "311", N_("whois (user)"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_311 },
   { "312", N_("whois (server)"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_312 },
   { "313", N_("whois (operator)"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_313 },
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_whois_nick_msg },
   { "314", N_("whowas"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_314 },
   { "315", N_("end of /who list"), "", "",
@@ -402,11 +336,11 @@ t_irc_command irc_commands[] =
   { "317", N_("whois (idle)"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_317 },
   { "318", N_("whois (end)"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_318 },
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_whois_nick_msg },
   { "319", N_("whois (channels)"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_319 },
   { "320", N_("whois (identified user)"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_320 },
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_whois_nick_msg },
   { "321", N_("/list start"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_321 },  
   { "322", N_("channel (for /list)"), "", "",
@@ -415,7 +349,9 @@ t_irc_command irc_commands[] =
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_323 },
   { "324", N_("channel mode"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_324 },
-  { "329", "???", "", "",
+  { "326", N_("whois (has oper privs)"), "", "",
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_whois_nick_msg },
+  { "329", N_("channel creation date"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_329 },
   { "331", N_("no topic for channel"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_331 },
@@ -426,6 +362,8 @@ t_irc_command irc_commands[] =
     NULL, 2, 0, MAX_ARGS, 1, NULL, NULL, irc_cmd_recv_332 },
   { "333", N_("infos about topic (nick and date changed)"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_333 },
+  { "338", N_("whois (host)"), "", "",
+    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_338 },
   { "341", N_("inviting"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_341 },
   { "344", N_("channel reop"), "", "",
@@ -445,40 +383,14 @@ t_irc_command irc_commands[] =
     N_("channel: name of channel\n"
        "   nick: nick on the channel"),
     NULL, 2, 0, MAX_ARGS, 1, NULL, NULL, irc_cmd_recv_353 },
-  { "364", N_("links"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "365", N_("end of /links list"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
   { "366", N_("end of /names list"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_366 },
   { "367", N_("banlist"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_367 },
   { "368", N_("end of banlist"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_368 },
-  { "369", N_("end of /whowas list"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "371", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "372", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "373", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "374", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "375", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "376", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
   { "378", N_("connecting from"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_378 },
-  { "379", N_("using modes"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "381", N_("you are now an IRC operator"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "382", N_("rehashing"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
-  { "391", N_("server local time"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
   { "401", N_("no such nick/channel"), "", "",
     NULL, 0, 0, MAX_ARGS, 1, NULL, NULL, irc_cmd_recv_error },
   { "402", N_("no such server"), "", "",
@@ -585,8 +497,6 @@ t_irc_command irc_commands[] =
     NULL, 0, 0, MAX_ARGS, 1, NULL, NULL, irc_cmd_recv_error },
   { "502", N_("can't change mode for other users"), "", "",
     NULL, 0, 0, MAX_ARGS, 1, NULL, NULL, irc_cmd_recv_error },
-  { "505", N_("a server message"), "", "",
-    NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_server_msg },
   { "671", N_("whois (secure connection)"), "", "",
     NULL, 0, 0, 0, 1, NULL, NULL, irc_cmd_recv_671 },
   { "973", N_("whois (secure connection)"), "", "",
