@@ -17,7 +17,7 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
  */
 
-/* irc-dcc.c: DCC communications (files & chat) */
+/* irc-dcc.c: Direct Client-to-Client (DCC) communication (files & chat) */
 
 
 #ifdef HAVE_CONFIG_H
@@ -26,12 +26,15 @@
 
 #include <stdlib.h>
 #include <unistd.h>
+#include <signal.h>
 #include <string.h>
 #include <stdarg.h>
 #include <fcntl.h>
 #include <time.h>
+#include <errno.h>
 #include <sys/stat.h>
 #include <sys/types.h>
+#include <sys/wait.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <arpa/inet.h>
@@ -41,11 +44,13 @@
 #include "irc.h"
 #include "../common/log.h"
 #include "../common/hotlist.h"
+#include "../common/util.h"
 #include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
 
 t_irc_dcc *dcc_list = NULL;     /* DCC files & chat list                    */
+t_irc_dcc *last_dcc = NULL;     /* last DCC in list                         */
 char *dcc_status_string[] =     /* strings for DCC status                   */
 { N_("Waiting"), N_("Connecting"), N_("Active"), N_("Done"), N_("Failed"),
   N_("Aborted") };
@@ -58,10 +63,13 @@ char *dcc_status_string[] =     /* strings for DCC status                   */
 void
 dcc_redraw (int highlight)
 {
-    gui_window_redraw_buffer (gui_buffer_get_dcc (gui_current_window));
-    if (highlight)
+    t_gui_buffer *ptr_buffer;
+
+    ptr_buffer = gui_buffer_get_dcc (gui_current_window);
+    gui_window_redraw_buffer (ptr_buffer);
+    if (highlight && gui_add_hotlist && (ptr_buffer->num_displayed == 0))
     {
-        hotlist_add (highlight, NULL, gui_buffer_get_dcc (gui_current_window));
+        hotlist_add (highlight, NULL, ptr_buffer, 0);
         gui_status_draw (gui_current_window->buffer, 0);
     }
 }
@@ -268,6 +276,49 @@ dcc_calculate_speed (t_irc_dcc *ptr_dcc, int ended)
     }
 }
 
+/*
+ * dcc_connect_to_sender: connect to sender
+ */
+
+int
+dcc_connect_to_sender (t_irc_dcc *ptr_dcc)
+{
+    struct sockaddr_in addr;
+    struct hostent *hostent;
+    char *ip4;
+    int ret;
+    
+    if (cfg_proxy_use)
+    {
+        memset (&addr, 0, sizeof (addr));
+        addr.sin_addr.s_addr = htonl (ptr_dcc->addr);
+        ip4 = inet_ntoa(addr.sin_addr);
+        
+        memset (&addr, 0, sizeof (addr));
+        addr.sin_port = htons (cfg_proxy_port);
+        addr.sin_family = AF_INET;
+        if ((hostent = gethostbyname (cfg_proxy_address)) == NULL)
+            return 0;
+        memcpy(&(addr.sin_addr),*(hostent->h_addr_list), sizeof(struct in_addr));
+        ret = connect (ptr_dcc->sock, (struct sockaddr *) &addr, sizeof (addr));
+        if ((ret == -1) && (errno != EINPROGRESS))
+            return 0;
+        if (pass_proxy (ptr_dcc->sock, ip4, ptr_dcc->port, ptr_dcc->server->username) == -1)
+            return 0;
+    }
+    else
+    {
+        memset (&addr, 0, sizeof (addr));
+        addr.sin_port = htons (ptr_dcc->port);
+        addr.sin_family = AF_INET;
+        addr.sin_addr.s_addr = htonl (ptr_dcc->addr);
+        ret = connect (ptr_dcc->sock, (struct sockaddr *) &addr, sizeof (addr));
+        if ((ret == -1) && (errno != EINPROGRESS))
+            return 0;
+    }
+    return 1;
+}
+
 /*
  * dcc_connect: connect to another host
  */
@@ -275,10 +326,6 @@ dcc_calculate_speed (t_irc_dcc *ptr_dcc, int ended)
 int
 dcc_connect (t_irc_dcc *ptr_dcc)
 {
-    struct sockaddr_in addr;
-    struct hostent *hostent;
-    char *ip4;
-    
     if (ptr_dcc->type == DCC_CHAT_SEND)
         ptr_dcc->status = DCC_WAITING;
     else
@@ -291,47 +338,26 @@ dcc_connect (t_irc_dcc *ptr_dcc)
             return 0;
     }
 
-    /* for sending (chat or file), listen to socket for a connection */
+    /* for chat or file sending, listen to socket for a connection */
     if (DCC_IS_SEND(ptr_dcc->type))
-      {
+    {
         if (fcntl (ptr_dcc->sock, F_SETFL, O_NONBLOCK) == -1)
-	  return 0;	
+            return 0;	
         if (listen (ptr_dcc->sock, 1) == -1)
             return 0;
         if (fcntl (ptr_dcc->sock, F_SETFL, 0) == -1)
             return 0;
     }
     
-    /* for receiving (chat or file), connect to listening host */
-    if (DCC_IS_RECV(ptr_dcc->type))
+    /* for chat receiving, connect to listening host */
+    if (ptr_dcc->type == DCC_CHAT_RECV)
     {
         if (fcntl (ptr_dcc->sock, F_SETFL, O_NONBLOCK) == -1)
-            return 0;      
-        if (cfg_proxy_use)
-	{
-            memset (&addr, 0, sizeof (addr));
-            addr.sin_addr.s_addr = htonl (ptr_dcc->addr);
-            ip4 = inet_ntoa(addr.sin_addr);
-
-            memset (&addr, 0, sizeof (addr));
-            addr.sin_port = htons (cfg_proxy_port);
-            addr.sin_family = AF_INET;
-            if ((hostent = gethostbyname (cfg_proxy_address)) == NULL)
-                return 0;
-            memcpy(&(addr.sin_addr),*(hostent->h_addr_list), sizeof(struct in_addr));
-            connect (ptr_dcc->sock, (struct sockaddr *) &addr, sizeof (addr));
-            if (pass_proxy(ptr_dcc->sock, ip4, ptr_dcc->port, ptr_dcc->server->username) == -1)
-                return 0;
-	}
-        else
-	{
-            memset (&addr, 0, sizeof (addr));
-            addr.sin_port = htons (ptr_dcc->port);
-            addr.sin_family = AF_INET;
-            addr.sin_addr.s_addr = htonl (ptr_dcc->addr);
-            connect (ptr_dcc->sock, (struct sockaddr *) &addr, sizeof (addr));
-	}
+            return 0;
+        dcc_connect_to_sender (ptr_dcc);
     }
+
+    /* for file receiving, connection is made in child process (blocking) socket */
     
     return 1;
 }
@@ -345,6 +371,9 @@ dcc_free (t_irc_dcc *ptr_dcc)
 {
     t_irc_dcc *new_dcc_list;
     
+    if (!ptr_dcc)
+        return;
+    
     /* DCC CHAT with channel => remove channel
        (to prevent channel from becoming standard pv) */
     if (ptr_dcc->channel)
@@ -354,11 +383,14 @@ dcc_free (t_irc_dcc *ptr_dcc)
             || (DCC_ENDED(((t_irc_dcc *)(ptr_dcc->channel->dcc_chat))->status)))
         {
             gui_buffer_free (ptr_dcc->channel->buffer, 1);
-            channel_free (ptr_dcc->server, ptr_dcc->channel);
+            if (ptr_dcc->channel)
+                channel_free (ptr_dcc->server, ptr_dcc->channel);
         }
     }
 
     /* remove DCC from list */
+    if (last_dcc == ptr_dcc)
+        last_dcc = ptr_dcc->prev_dcc;
     if (ptr_dcc->prev_dcc)
     {
         (ptr_dcc->prev_dcc)->next_dcc = ptr_dcc->next_dcc;
@@ -381,6 +413,34 @@ dcc_free (t_irc_dcc *ptr_dcc)
     dcc_list = new_dcc_list;
 }
 
+/*
+ * dcc_file_child_kill: kill child process and close pipe
+ */
+
+void
+dcc_file_child_kill (t_irc_dcc *ptr_dcc)
+{
+    /* kill process */
+    if (ptr_dcc->child_pid > 0)
+    {
+        kill (ptr_dcc->child_pid, SIGKILL);
+        waitpid (ptr_dcc->child_pid, NULL, 0);
+        ptr_dcc->child_pid = 0;
+    }
+    
+    /* close pipe used with child */
+    if (ptr_dcc->child_read != -1)
+    {
+        close (ptr_dcc->child_read);
+        ptr_dcc->child_read = -1;
+    }
+    if (ptr_dcc->child_write != -1)
+    {
+        close (ptr_dcc->child_write);
+        ptr_dcc->child_write = -1;
+    }
+}
+
 /*
  * dcc_close: close a DCC connection
  */
@@ -419,6 +479,7 @@ dcc_close (t_irc_dcc *ptr_dcc, int status)
                         ptr_dcc->nick,
                         GUI_COLOR(COLOR_WIN_CHAT),
                         (status == DCC_DONE) ? _("OK") : _("FAILED"));
+            dcc_file_child_kill (ptr_dcc);
         }
     }
     if (status == DCC_ABORTED)
@@ -543,24 +604,16 @@ dcc_recv_connect_init (t_irc_dcc *ptr_dcc)
     }
     else
     {
-        ptr_dcc->status = DCC_ACTIVE;
-        
-        /* DCC file => look for local filename and open it in writing mode */
+        /* DCC file => launch child process */
         if (DCC_IS_FILE(ptr_dcc->type))
         {
-            if (ptr_dcc->start_resume > 0)
-                ptr_dcc->file = open (ptr_dcc->local_filename,
-                                      O_APPEND | O_WRONLY | O_NONBLOCK);
-            else
-                ptr_dcc->file = open (ptr_dcc->local_filename,
-                                      O_CREAT | O_TRUNC | O_WRONLY | O_NONBLOCK,
-                                      0644);
-            ptr_dcc->start_transfer = time (NULL);
-            ptr_dcc->last_check_time = time (NULL);
+            ptr_dcc->status = DCC_CONNECTING;
+            dcc_file_recv_fork (ptr_dcc);
         }
         else
         {
             /* DCC CHAT => associate DCC with channel */
+            ptr_dcc->status = DCC_ACTIVE;
             dcc_channel_for_chat (ptr_dcc);
         }
     }
@@ -678,11 +731,16 @@ dcc_alloc ()
     new_dcc->port = 0;
     new_dcc->nick = NULL;
     new_dcc->sock = -1;
+    new_dcc->child_pid = 0;
+    new_dcc->child_read = -1;
+    new_dcc->child_write = -1;
     new_dcc->unterminated_message = NULL;
+    new_dcc->fast_send = cfg_dcc_fast_send;
     new_dcc->file = -1;
     new_dcc->filename = NULL;
     new_dcc->local_filename = NULL;
     new_dcc->filename_suffix = -1;
+    new_dcc->blocksize = cfg_dcc_blocksize;
     new_dcc->size = 0;
     new_dcc->pos = 0;
     new_dcc->ack = 0;
@@ -697,6 +755,8 @@ dcc_alloc ()
     new_dcc->next_dcc = dcc_list;
     if (dcc_list)
         dcc_list->prev_dcc = new_dcc;
+    else
+        last_dcc = new_dcc;
     dcc_list = new_dcc;
     
     return new_dcc;
@@ -1179,11 +1239,26 @@ dcc_chat_sendf (t_irc_dcc *ptr_dcc, char *fmt, ...)
 void
 dcc_chat_recv (t_irc_dcc *ptr_dcc)
 {
+    fd_set read_fd;
+    static struct timeval timeout;
     static char buffer[4096 + 2];
     char *buf2, *pos, *ptr_buf, *ptr_buf2, *next_ptr_buf;
     char *ptr_buf_color;
     int num_read;
+    
+    FD_ZERO (&read_fd);
+    FD_SET (ptr_dcc->sock, &read_fd);
+    timeout.tv_sec = 0;
+    timeout.tv_usec = 0;
+    
+    /* something to read on socket? */
+    if (select (FD_SETSIZE, &read_fd, NULL, NULL, &timeout) <= 0)
+        return;
 
+    if (!FD_ISSET (ptr_dcc->sock, &read_fd))
+        return;
+
+    /* there's something to read on socket! */
     num_read = recv (ptr_dcc->sock, buffer, sizeof (buffer) - 2, 0);
     if (num_read > 0)
     {
@@ -1280,6 +1355,397 @@ dcc_chat_recv (t_irc_dcc *ptr_dcc)
     }
 }
 
+/*
+ * dcc_file_create_pipe: create pipe for communication with child process
+ *                       return 1 if ok, 0 if error
+ */
+
+int
+dcc_file_create_pipe (t_irc_dcc *ptr_dcc)
+{
+    int child_pipe[2];
+    
+    if (pipe (child_pipe) < 0)
+    {
+        irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                            PREFIX_ERROR);
+        gui_printf (ptr_dcc->server->buffer,
+                    _("%s DCC: unable to create pipe\n"),
+                    WEECHAT_ERROR);
+        dcc_close (ptr_dcc, DCC_FAILED);
+        dcc_redraw (HOTLIST_MSG);
+        return 0;
+    }
+    
+    ptr_dcc->child_read = child_pipe[0];
+    ptr_dcc->child_write = child_pipe[1];
+    return 1;
+}
+
+/*
+ * dcc_file_write_pipe: write data into pipe
+ */
+
+void
+dcc_file_write_pipe (t_irc_dcc *ptr_dcc, int status, int error)
+{
+    char buffer[1 + 1 + 12 + 1];   /* status + error + pos + \0 */
+
+    snprintf (buffer, sizeof (buffer), "%c%c%012lu",
+              status + '0', error + '0', ptr_dcc->pos);
+    write (ptr_dcc->child_write, buffer, sizeof (buffer));
+}
+
+/*
+ * dcc_file_send_child: child process for sending file
+ */
+
+void
+dcc_file_send_child (t_irc_dcc *ptr_dcc)
+{
+    int num_read, num_sent;
+    static char buffer[DCC_MAX_BLOCKSIZE];
+    uint32_t ack;
+    time_t last_sent, new_time;
+    
+    last_sent = time (NULL);
+    while (1)
+    {
+        /* read DCC ACK (sent by receiver) */
+        if (ptr_dcc->pos > ptr_dcc->ack)
+        {
+            /* we should receive ACK for packets sent previously */
+            while (1)
+            {
+                num_read = recv (ptr_dcc->sock, (char *) &ack, 4, MSG_PEEK);
+                if ((num_read < 1) &&
+                    ((num_read != -1) || (errno != EAGAIN)))
+                {
+                    dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_SEND_BLOCK);
+                    return;
+                }
+                if (num_read == 4)
+                {
+                    recv (ptr_dcc->sock, (char *) &ack, 4, 0);
+                    ptr_dcc->ack = ntohl (ack);
+                    
+                    /* DCC send ok? */
+                    if ((ptr_dcc->pos >= ptr_dcc->size)
+                        && (ptr_dcc->ack >= ptr_dcc->size))
+                    {
+                        dcc_file_write_pipe (ptr_dcc, DCC_DONE, DCC_NO_ERROR);
+                        return;
+                    }
+                }
+                else
+                    break;
+            }
+        }
+        
+        /* send a block to receiver */
+        if ((ptr_dcc->pos < ptr_dcc->size) &&
+             (ptr_dcc->fast_send || (ptr_dcc->pos <= ptr_dcc->ack)))
+        {
+            lseek (ptr_dcc->file, ptr_dcc->pos, SEEK_SET);
+            num_read = read (ptr_dcc->file, buffer, ptr_dcc->blocksize);
+            if (num_read < 1)
+            {
+                dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_READ_LOCAL);
+                return;
+            }
+            num_sent = send (ptr_dcc->sock, buffer, num_read, 0);
+            if (num_sent < 0)
+            {
+                /* socket is temporarily not available (receiver can't receive
+                   amount of data we sent ?!) */
+                if (errno == EAGAIN)
+                    usleep (1000);
+                else
+                {
+                    dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_SEND_BLOCK);
+                    return;
+                }
+            }
+            if (num_sent > 0)
+            {
+                ptr_dcc->pos += (unsigned long) num_sent;
+                new_time = time (NULL);
+                if (last_sent != new_time)
+                {
+                    last_sent = new_time;
+                    dcc_file_write_pipe (ptr_dcc, DCC_ACTIVE, DCC_NO_ERROR);
+                }
+            }
+        }
+        else
+            usleep (1000);
+    }
+}
+
+/*
+ * dcc_file_recv_child: child process for receiving file
+ */
+
+void
+dcc_file_recv_child (t_irc_dcc *ptr_dcc)
+{
+    int num_read;
+    static char buffer[DCC_MAX_BLOCKSIZE];
+    uint32_t pos;
+    time_t last_sent, new_time;
+    
+    /* first connect to sender (blocking) */
+    if (!dcc_connect_to_sender (ptr_dcc))
+    {
+        dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_CONNECT_SENDER);
+        return;
+    }
+
+    /* connection is ok, change DCC status (inform parent process) */
+    dcc_file_write_pipe (ptr_dcc, DCC_ACTIVE, DCC_NO_ERROR);
+    
+    last_sent = time (NULL);
+    while (1)
+    {    
+        num_read = recv (ptr_dcc->sock, buffer, sizeof (buffer), 0);
+        if (num_read == -1)
+        {
+            /* socket is temporarily not available (sender is not fast ?!) */
+            if (errno == EAGAIN)
+                usleep (1000);
+            else
+            {
+                dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_RECV_BLOCK);
+                return;
+            }
+        }
+        else
+        {
+            if (num_read == 0)
+            {
+                dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_RECV_BLOCK);
+                return;
+            }
+            
+            if (write (ptr_dcc->file, buffer, num_read) == -1)
+            {
+                dcc_file_write_pipe (ptr_dcc, DCC_FAILED, DCC_ERROR_WRITE_LOCAL);
+                return;
+            }
+            
+            ptr_dcc->pos += (unsigned long) num_read;
+            pos = htonl (ptr_dcc->pos);
+            
+            /* we don't check return code, not a problem if an ACK send failed */
+            send (ptr_dcc->sock, (char *) &pos, 4, 0);
+
+            /* file received ok? */
+            if (ptr_dcc->pos >= ptr_dcc->size)
+            {
+                dcc_file_write_pipe (ptr_dcc, DCC_DONE, DCC_NO_ERROR);
+                return;
+            }
+            
+            new_time = time (NULL);
+            if (last_sent != new_time)
+            {
+                last_sent = new_time;
+                dcc_file_write_pipe (ptr_dcc, DCC_ACTIVE, DCC_NO_ERROR);
+            }
+        }
+    }
+}
+
+/*
+ * dcc_file_child_read: read data from child via pipe
+ */
+
+void
+dcc_file_child_read (t_irc_dcc *ptr_dcc)
+{
+    fd_set read_fd;
+    static struct timeval timeout;
+    char bufpipe[1 + 1 + 12 + 1];
+    int num_read;
+    char *error;
+    
+    FD_ZERO (&read_fd);
+    FD_SET (ptr_dcc->child_read, &read_fd);
+    timeout.tv_sec = 0;
+    timeout.tv_usec = 0;
+    
+    /* something to read on child pipe? */
+    if (select (FD_SETSIZE, &read_fd, NULL, NULL, &timeout) <= 0)
+        return;
+
+    if (!FD_ISSET (ptr_dcc->child_read, &read_fd))
+        return;
+
+    /* there's something to read in pipe! */
+    num_read = read (ptr_dcc->child_read, bufpipe, sizeof (bufpipe));
+    if (num_read > 0)
+    {
+        error = NULL;
+        ptr_dcc->pos = strtol (bufpipe + 2, &error, 10);
+        ptr_dcc->last_activity = time (NULL);
+        dcc_calculate_speed (ptr_dcc, 0);
+        
+        /* read error code */
+        switch (bufpipe[1] - '0')
+        {
+            /* errors for sender */
+            case DCC_ERROR_READ_LOCAL:
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: unable to read local file\n"),
+                            WEECHAT_ERROR);
+                break;
+            case DCC_ERROR_SEND_BLOCK:
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: unable to send block to receiver\n"),
+                            WEECHAT_ERROR);
+                break;
+            case DCC_ERROR_READ_ACK:
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: unable to read ACK from receiver\n"),
+                            WEECHAT_ERROR);
+                break;
+            /* errors for receiver */
+            case DCC_ERROR_CONNECT_SENDER:
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: unable to connect to sender\n"),
+                            WEECHAT_ERROR);
+                break;
+            case DCC_ERROR_RECV_BLOCK:
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: unable to receive block from sender\n"),
+                            WEECHAT_ERROR);
+                break;
+            case DCC_ERROR_WRITE_LOCAL:
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: unable to write local file\n"),
+                            WEECHAT_ERROR);
+                break;
+        }
+        
+        /* read new DCC status */
+        switch (bufpipe[0] - '0')
+        {
+            case DCC_ACTIVE:
+                if (ptr_dcc->status == DCC_CONNECTING)
+                {
+                    /* connection was successful by child, init transfert times */
+                    ptr_dcc->status = DCC_ACTIVE;
+                    ptr_dcc->start_transfer = time (NULL);
+                    ptr_dcc->last_check_time = time (NULL);
+                    dcc_redraw (HOTLIST_MSG);
+                }
+                else
+                    dcc_redraw (HOTLIST_LOW);
+                break;
+            case DCC_DONE:
+                dcc_close (ptr_dcc, DCC_DONE);
+                dcc_redraw (HOTLIST_MSG);
+                break;
+            case DCC_FAILED:
+                dcc_close (ptr_dcc, DCC_FAILED);
+                dcc_redraw (HOTLIST_MSG);
+                break;
+        }
+    }
+}
+
+/*
+ * dcc_file_send_fork: fork process for sending file
+ */
+
+void
+dcc_file_send_fork (t_irc_dcc *ptr_dcc)
+{
+    pid_t pid;
+    
+    if (!dcc_file_create_pipe (ptr_dcc))
+        return;
+
+    ptr_dcc->file = open (ptr_dcc->local_filename, O_RDONLY | O_NONBLOCK, 0644);
+    
+    switch (pid = fork ())
+    {
+        /* fork failed */
+        case -1:
+            irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                PREFIX_ERROR);
+            gui_printf (ptr_dcc->server->buffer,
+                        _("%s DCC: unable to fork\n"),
+                        WEECHAT_ERROR);
+            dcc_close (ptr_dcc, DCC_FAILED);
+            dcc_redraw (HOTLIST_MSG);
+            return;
+            /* child process */
+        case 0:
+            setuid (getuid ());
+            dcc_file_send_child (ptr_dcc);
+            _exit (EXIT_SUCCESS);
+    }
+    
+    /* parent process */
+    ptr_dcc->child_pid = pid;
+}
+
+/*
+ * dcc_file_recv_fork: fork process for receiving file
+ */
+
+void
+dcc_file_recv_fork (t_irc_dcc *ptr_dcc)
+{
+    pid_t pid;
+    
+    if (!dcc_file_create_pipe (ptr_dcc))
+        return;
+    
+    if (ptr_dcc->start_resume > 0)
+        ptr_dcc->file = open (ptr_dcc->local_filename,
+                              O_APPEND | O_WRONLY | O_NONBLOCK);
+    else
+        ptr_dcc->file = open (ptr_dcc->local_filename,
+                              O_CREAT | O_TRUNC | O_WRONLY | O_NONBLOCK,
+                              0644);
+    
+    switch (pid = fork ())
+    {
+        /* fork failed */
+        case -1:
+            irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                PREFIX_ERROR);
+            gui_printf (ptr_dcc->server->buffer,
+                        _("%s DCC: unable to fork\n"),
+                        WEECHAT_ERROR);
+            dcc_close (ptr_dcc, DCC_FAILED);
+            dcc_redraw (HOTLIST_MSG);
+            return;
+            /* child process */
+        case 0:
+            setuid (getuid ());
+            dcc_file_recv_child (ptr_dcc);
+            _exit (EXIT_SUCCESS);
+    }
+    
+    /* parent process */
+    ptr_dcc->child_pid = pid;
+}
+
 /*
  * dcc_handle: receive/send data for all active DCC
  */
@@ -1288,9 +1754,6 @@ void
 dcc_handle ()
 {
     t_irc_dcc *ptr_dcc;
-    int num_read, num_sent;
-    static char buffer[102400];
-    uint32_t pos;
     fd_set read_fd;
     static struct timeval timeout;
     int sock;
@@ -1304,6 +1767,11 @@ dcc_handle ()
         {
             if ((cfg_dcc_timeout != 0) && (time (NULL) > ptr_dcc->last_activity + cfg_dcc_timeout))
             {
+                irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                    PREFIX_ERROR);
+                gui_printf (ptr_dcc->server->buffer,
+                            _("%s DCC: timeout\n"),
+                            WEECHAT_ERROR);
                 dcc_close (ptr_dcc, DCC_FAILED);
                 dcc_redraw (HOTLIST_MSG);
                 continue;
@@ -1331,6 +1799,11 @@ dcc_handle ()
                         ptr_dcc->sock = -1;
                         if (sock < 0)
                         {
+                            irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                                PREFIX_ERROR);
+                            gui_printf (ptr_dcc->server->buffer,
+                                        _("%s DCC: unable to create socket for sending file\n"),
+                                        WEECHAT_ERROR);
                             dcc_close (ptr_dcc, DCC_FAILED);
                             dcc_redraw (HOTLIST_MSG);
                             continue;
@@ -1338,18 +1811,28 @@ dcc_handle ()
                         ptr_dcc->sock = sock;
                         if (fcntl (ptr_dcc->sock, F_SETFL, O_NONBLOCK) == -1)
                         {
+                            irc_display_prefix (ptr_dcc->server, ptr_dcc->server->buffer,
+                                                PREFIX_ERROR);
+                            gui_printf (ptr_dcc->server->buffer,
+                                        _("%s DCC: unable to set 'nonblock' option for socket\n"),
+                                        WEECHAT_ERROR);
                             dcc_close (ptr_dcc, DCC_FAILED);
                             dcc_redraw (HOTLIST_MSG);
                             continue;
                         }
                         ptr_dcc->addr = ntohl (addr.sin_addr.s_addr);
                         ptr_dcc->status = DCC_ACTIVE;
-                        ptr_dcc->file = open (ptr_dcc->local_filename, O_RDONLY | O_NONBLOCK, 0644);
                         ptr_dcc->start_transfer = time (NULL);
                         dcc_redraw (HOTLIST_MSG);
+                        dcc_file_send_fork (ptr_dcc);
                     }
                 }
             }
+            if (ptr_dcc->type == DCC_FILE_RECV)
+            {
+                if (ptr_dcc->child_read != -1)
+                    dcc_file_child_read (ptr_dcc);
+            }
         }
         
         if (ptr_dcc->status == DCC_WAITING)
@@ -1395,113 +1878,9 @@ dcc_handle ()
         if (ptr_dcc->status == DCC_ACTIVE)
         {
             if (DCC_IS_CHAT(ptr_dcc->type))
-            {
-                FD_ZERO (&read_fd);
-                FD_SET (ptr_dcc->sock, &read_fd);
-                timeout.tv_sec = 0;
-                timeout.tv_usec = 0;
-                
-                /* something to read on socket? */
-                if (select (FD_SETSIZE, &read_fd, NULL, NULL, &timeout) > 0)
-                {
-                    if (FD_ISSET (ptr_dcc->sock, &read_fd))
-                        dcc_chat_recv (ptr_dcc);
-                }
-            }
-            if (ptr_dcc->type == DCC_FILE_RECV)
-            {
-                num_read = recv (ptr_dcc->sock, buffer, sizeof (buffer), 0);
-                if (num_read != -1)
-                {
-                    if (num_read == 0)
-                    {
-                        dcc_close (ptr_dcc, DCC_FAILED);
-                        dcc_redraw (HOTLIST_MSG);
-                        continue;
-                    }
-                    
-                    if (write (ptr_dcc->file, buffer, num_read) == -1)
-                    {
-                        dcc_close (ptr_dcc, DCC_FAILED);
-                        dcc_redraw (HOTLIST_MSG);
-                        continue;
-                    }
-                    ptr_dcc->last_activity = time (NULL);
-                    ptr_dcc->pos += (unsigned long) num_read;
-                    pos = htonl (ptr_dcc->pos);
-                    send (ptr_dcc->sock, (char *) &pos, 4, 0);
-                    dcc_calculate_speed (ptr_dcc, 0);
-                    if (ptr_dcc->pos >= ptr_dcc->size)
-                    {
-                        dcc_close (ptr_dcc, DCC_DONE);
-                        dcc_redraw (HOTLIST_MSG);
-                    }
-                    else
-                        dcc_redraw (HOTLIST_LOW);
-                }
-            }
-            if (ptr_dcc->type == DCC_FILE_SEND)
-            {
-                if (cfg_dcc_blocksize > (int) sizeof (buffer))
-                {
-                    irc_display_prefix (NULL, NULL, PREFIX_ERROR);
-                    gui_printf (NULL, _("%s DCC failed because blocksize is too "
-                                "big. Check value of \"dcc_blocksize\" option, "
-                                "max is %d.\n"),
-                                sizeof (buffer));
-                    dcc_close (ptr_dcc, DCC_FAILED);
-                    dcc_redraw (HOTLIST_MSG);
-                    continue;
-                }
-                if (ptr_dcc->pos > ptr_dcc->ack)
-                {
-                    /* we should receive ACK for packets sent previously */
-                    num_read = recv (ptr_dcc->sock, (char *) &pos, 4, MSG_PEEK);
-                    if (num_read != -1)
-                    {
-                        if (num_read == 0)
-                        {
-                            dcc_close (ptr_dcc, DCC_FAILED);
-                            dcc_redraw (HOTLIST_MSG);
-                            continue;
-                        }
-                        if (num_read < 4)
-                            continue;
-                        recv (ptr_dcc->sock, (char *) &pos, 4, 0);
-                        ptr_dcc->ack = ntohl (pos);
-                        
-                        if ((ptr_dcc->pos >= ptr_dcc->size)
-                            && (ptr_dcc->ack >= ptr_dcc->size))
-                        {
-                            dcc_close (ptr_dcc, DCC_DONE);
-                            dcc_redraw (HOTLIST_MSG);
-                            continue;
-                        }
-                    }
-                }
-                if (ptr_dcc->pos <= ptr_dcc->ack)
-                {
-                    lseek (ptr_dcc->file, ptr_dcc->pos, SEEK_SET);
-                    num_read = read (ptr_dcc->file, buffer, cfg_dcc_blocksize);
-                    if (num_read < 1)
-                    {
-                        dcc_close (ptr_dcc, DCC_FAILED);
-                        dcc_redraw (HOTLIST_MSG);
-                        continue;
-                    }
-                    num_sent = send (ptr_dcc->sock, buffer, num_read, 0);
-                    if (num_sent < 0)
-                    {
-                        dcc_close (ptr_dcc, DCC_FAILED);
-                        dcc_redraw (HOTLIST_MSG);
-                        continue;
-                    }
-                    ptr_dcc->last_activity = time (NULL);
-                    ptr_dcc->pos += (unsigned long) num_sent;
-                    dcc_calculate_speed (ptr_dcc, 0);
-                    dcc_redraw (HOTLIST_LOW);
-                }
-            }
+                dcc_chat_recv (ptr_dcc);
+            else
+                dcc_file_child_read (ptr_dcc);
         }
     }
 }
@@ -1532,33 +1911,44 @@ dcc_end ()
  */
 
 void
-dcc_print_log (t_irc_dcc *dcc)
+dcc_print_log ()
 {
-    weechat_log_printf ("[DCC (addr:0x%X)]\n", dcc);
-    weechat_log_printf ("  server. . . . . . . : 0x%X\n", dcc->server);
-    weechat_log_printf ("  channel . . . . . . : 0x%X\n", dcc->channel);
-    weechat_log_printf ("  type. . . . . . . . : %d\n",   dcc->type);
-    weechat_log_printf ("  status. . . . . . . : %d\n",   dcc->status);
-    weechat_log_printf ("  start_time. . . . . : %ld\n",  dcc->start_time);
-    weechat_log_printf ("  start_transfer. . . : %ld\n",  dcc->start_transfer);
-    weechat_log_printf ("  addr. . . . . . . . : %lu\n",  dcc->addr);
-    weechat_log_printf ("  port. . . . . . . . : %d\n",   dcc->port);
-    weechat_log_printf ("  nick. . . . . . . . : '%s'\n", dcc->nick);
-    weechat_log_printf ("  sock. . . . . . . . : %d\n",   dcc->sock);
-    weechat_log_printf ("  unterminated_message: '%s'\n", dcc->unterminated_message);
-    weechat_log_printf ("  file. . . . . . . . : %d\n",   dcc->file);
-    weechat_log_printf ("  filename. . . . . . : '%s'\n", dcc->filename);
-    weechat_log_printf ("  local_filename. . . : '%s'\n", dcc->local_filename);
-    weechat_log_printf ("  filename_suffix . . : %d\n",   dcc->filename_suffix);
-    weechat_log_printf ("  size. . . . . . . . : %lu\n",  dcc->size);
-    weechat_log_printf ("  pos . . . . . . . . : %lu\n",  dcc->pos);
-    weechat_log_printf ("  ack . . . . . . . . : %lu\n",  dcc->ack);
-    weechat_log_printf ("  start_resume. . . . : %lu\n",  dcc->start_resume);
-    weechat_log_printf ("  last_check_time . . : %ld\n",  dcc->last_check_time);
-    weechat_log_printf ("  last_check_pos. . . : %lu\n",  dcc->last_check_pos);
-    weechat_log_printf ("  last_activity . . . : %ld\n",  dcc->last_activity);
-    weechat_log_printf ("  bytes_per_sec . . . : %lu\n",  dcc->bytes_per_sec);
-    weechat_log_printf ("  eta . . . . . . . . : %lu\n",  dcc->eta);
-    weechat_log_printf ("  prev_dcc. . . . . . : 0x%X\n", dcc->prev_dcc);
-    weechat_log_printf ("  next_dcc. . . . . . : 0x%X\n", dcc->next_dcc);
+    t_irc_dcc *ptr_dcc;
+    
+    for (ptr_dcc = dcc_list; ptr_dcc; ptr_dcc = ptr_dcc->next_dcc)
+    {
+        weechat_log_printf ("\n");
+        weechat_log_printf ("[DCC (addr:0x%X)]\n", ptr_dcc);
+        weechat_log_printf ("  server. . . . . . . : 0x%X\n", ptr_dcc->server);
+        weechat_log_printf ("  channel . . . . . . : 0x%X\n", ptr_dcc->channel);
+        weechat_log_printf ("  type. . . . . . . . : %d\n",   ptr_dcc->type);
+        weechat_log_printf ("  status. . . . . . . : %d\n",   ptr_dcc->status);
+        weechat_log_printf ("  start_time. . . . . : %ld\n",  ptr_dcc->start_time);
+        weechat_log_printf ("  start_transfer. . . : %ld\n",  ptr_dcc->start_transfer);
+        weechat_log_printf ("  addr. . . . . . . . : %lu\n",  ptr_dcc->addr);
+        weechat_log_printf ("  port. . . . . . . . : %d\n",   ptr_dcc->port);
+        weechat_log_printf ("  nick. . . . . . . . : '%s'\n", ptr_dcc->nick);
+        weechat_log_printf ("  sock. . . . . . . . : %d\n",   ptr_dcc->sock);
+        weechat_log_printf ("  child_pid . . . . . : %d\n",   ptr_dcc->child_pid);
+        weechat_log_printf ("  child_read. . . . . : %d\n",   ptr_dcc->child_read);
+        weechat_log_printf ("  child_write . . . . : %d\n",   ptr_dcc->child_write);
+        weechat_log_printf ("  unterminated_message: '%s'\n", ptr_dcc->unterminated_message);
+        weechat_log_printf ("  fast_send . . . . . : %d\n",   ptr_dcc->fast_send);
+        weechat_log_printf ("  file. . . . . . . . : %d\n",   ptr_dcc->file);
+        weechat_log_printf ("  filename. . . . . . : '%s'\n", ptr_dcc->filename);
+        weechat_log_printf ("  local_filename. . . : '%s'\n", ptr_dcc->local_filename);
+        weechat_log_printf ("  filename_suffix . . : %d\n",   ptr_dcc->filename_suffix);
+        weechat_log_printf ("  blocksize . . . . . : %d\n",   ptr_dcc->blocksize);
+        weechat_log_printf ("  size. . . . . . . . : %lu\n",  ptr_dcc->size);
+        weechat_log_printf ("  pos . . . . . . . . : %lu\n",  ptr_dcc->pos);
+        weechat_log_printf ("  ack . . . . . . . . : %lu\n",  ptr_dcc->ack);
+        weechat_log_printf ("  start_resume. . . . : %lu\n",  ptr_dcc->start_resume);
+        weechat_log_printf ("  last_check_time . . : %ld\n",  ptr_dcc->last_check_time);
+        weechat_log_printf ("  last_check_pos. . . : %lu\n",  ptr_dcc->last_check_pos);
+        weechat_log_printf ("  last_activity . . . : %ld\n",  ptr_dcc->last_activity);
+        weechat_log_printf ("  bytes_per_sec . . . : %lu\n",  ptr_dcc->bytes_per_sec);
+        weechat_log_printf ("  eta . . . . . . . . : %lu\n",  ptr_dcc->eta);
+        weechat_log_printf ("  prev_dcc. . . . . . : 0x%X\n", ptr_dcc->prev_dcc);
+        weechat_log_printf ("  next_dcc. . . . . . : 0x%X\n", ptr_dcc->next_dcc);
+    }
 }
