@@ -39,6 +39,7 @@
 #include "../common/weechat.h"
 #include "irc.h"
 #include "../common/command.h"
+#include "../common/util.h"
 #include "../common/weeconfig.h"
 #include "../gui/gui.h"
 
@@ -243,6 +244,11 @@ irc_send_away (t_irc_server *server, char *arguments)
     if (arguments)
     {
         server->is_away = 1;
+        if (server->away_message)
+            free (server->away_message);
+        server->away_message = (char *) malloc (strlen (arguments) + 1);
+        if (server->away_message)
+            strcpy (server->away_message, arguments);
         server->away_time = time (NULL);
         server_sendf (server, "AWAY :%s\r\n", arguments);
         if (cfg_irc_display_away != CFG_IRC_DISPLAY_AWAY_OFF)
@@ -271,6 +277,11 @@ irc_send_away (t_irc_server *server, char *arguments)
     {
         server_sendf (server, "AWAY\r\n");
         server->is_away = 0;
+        if (server->away_message)
+        {
+            free (server->away_message);
+            server->away_message = NULL;
+        }
         if (server->away_time != 0)
         {
             time_now = time (NULL);
@@ -337,7 +348,18 @@ irc_cmd_send_away (t_irc_server *server, t_irc_channel *channel,
         }
     }
     else
-        irc_send_away (server, arguments);
+    {
+        if (server->is_connected)
+            irc_send_away (server, arguments);
+        else
+        {
+            irc_display_prefix (NULL, NULL, PREFIX_ERROR);
+            gui_printf_nolog (NULL,
+                              _("%s command \"%s\" needs a server connection!\n"),
+                              WEECHAT_ERROR, "away");
+            return -1;
+        }
+    }
     
     gui_status_draw (buffer, 1);
     gui_add_hotlist = 1;
@@ -1531,17 +1553,25 @@ irc_cmd_send_part (t_irc_server *server, t_irc_channel *channel,
         }
         else
         {
+            if (!CHANNEL(buffer))
+            {
+                irc_display_prefix (NULL, server->buffer, PREFIX_ERROR);
+                gui_printf_nolog (server->buffer,
+                                  _("%s \"%s\" command can only be executed in a channel or private buffer\n"),
+                                  WEECHAT_ERROR, "part");
+                return -1;
+            }
             channel_name = CHANNEL(buffer)->name;
             pos_args = arguments;
         }
     }
     else
     {
-        if (BUFFER_IS_SERVER(buffer))
+        if (!CHANNEL(buffer))
         {
             irc_display_prefix (NULL, server->buffer, PREFIX_ERROR);
             gui_printf_nolog (server->buffer,
-                              _("%s \"%s\" command can not be executed on a server buffer\n"),
+                              _("%s \"%s\" command can only be executed in a channel or private buffer\n"),
                               WEECHAT_ERROR, "part");
             return -1;
         }
@@ -1680,6 +1710,34 @@ irc_cmd_send_query (t_irc_server *server, t_irc_channel *channel,
     return 0;
 }
 
+/*
+ * irc_send_quit_server: send QUIT to a server
+ */
+
+void
+irc_send_quit_server (t_irc_server *server, char *arguments)
+{
+    char *ptr_arg, *buf;
+    
+    if (server->is_connected)
+    {
+        ptr_arg = (arguments) ? arguments :
+            (cfg_irc_default_msg_quit && cfg_irc_default_msg_quit[0]) ?
+            cfg_irc_default_msg_quit : NULL;
+        
+        if (ptr_arg)
+        {
+            buf = weechat_strreplace (ptr_arg, "%v", PACKAGE_VERSION);
+            server_sendf (server, "QUIT :%s\r\n",
+                          (buf) ? buf : ptr_arg);
+            if (buf)
+                free (buf);
+        }
+        else
+            server_sendf (server, "QUIT\r\n");
+    }
+}
+
 /*
  * irc_cmd_send_quit: disconnect from all servers and quit WeeChat
  */
@@ -1689,31 +1747,15 @@ irc_cmd_send_quit (t_irc_server *server, t_irc_channel *channel,
                    char *arguments)
 {
     t_irc_server *ptr_server;
-    char *ptr_arg, *buf;
     
     /* make gcc happy */
     (void) server;
     (void) channel;
     
-    ptr_arg = (arguments) ? arguments :
-              (cfg_irc_default_msg_quit && cfg_irc_default_msg_quit[0]) ?
-              cfg_irc_default_msg_quit : NULL;
     for (ptr_server = irc_servers; ptr_server;
          ptr_server = ptr_server->next_server)
     {
-        if (ptr_server->is_connected)
-        {
-            if (ptr_arg)
-            {
-                buf = weechat_strreplace (ptr_arg, "%v", PACKAGE_VERSION);
-                server_sendf (ptr_server, "QUIT :%s\r\n",
-                              (buf) ? buf : ptr_arg);
-                if (buf)
-                    free (buf);
-            }
-            else
-                server_sendf (ptr_server, "QUIT\r\n");
-        }
+        irc_send_quit_server (ptr_server, arguments);
     }
     quit_weechat = 1;
     return 0;
@@ -2169,7 +2211,7 @@ irc_cmd_send_wallops (t_irc_server *server, t_irc_channel *channel,
     /* make gcc happy */
     (void) channel;
     
-    server_sendf (server, "WALLOPS %s\r\n", arguments);
+    server_sendf (server, "WALLOPS :%s\r\n", arguments);
     return 0;
 }
 
