@@ -102,12 +102,11 @@ server_init (t_irc_server *server)
     server->ssl_connected = 0;
     server->unterminated_message = NULL;
     server->nick = NULL;
-    server->nick_modes = (char *) malloc (NUM_NICK_MODES + 1);
-    memset (server->nick_modes, ' ', NUM_NICK_MODES);
-    server->nick_modes[NUM_NICK_MODES] = '\0';
+    server->nick_modes = NULL;
     server->reconnect_start = 0;
     server->reconnect_join = 0;
     server->is_away = 0;
+    server->away_message = NULL;
     server->away_time = 0;
     server->lag = 0;
     server->lag_check_time.tv_sec = 0;
@@ -283,6 +282,9 @@ server_alloc ()
 void
 server_destroy (t_irc_server *server)
 {
+    if (!server)
+        return;
+    
     /* free data */
     if (server->name)
         free (server->name);
@@ -320,6 +322,8 @@ server_destroy (t_irc_server *server)
         free (server->nick);
     if (server->nick_modes)
         free (server->nick_modes);
+    if (server->away_message)
+        free (server->away_message);
     if (server->channels)
         channel_free_all (server);
 }
@@ -333,6 +337,9 @@ server_free (t_irc_server *server)
 {
     t_irc_server *new_irc_servers;
 
+    if (!server)
+        return;
+    
     /* close any opened channel/private */
     while (server->channels)
         channel_free (server, server->channels);
@@ -759,9 +766,14 @@ server_msgq_flush ()
                 if (ptr_data[0] == ':')
                 {
                     pos = strchr (ptr_data, ' ');
-                    pos[0] = '\0';
-                    host = ptr_data + 1;
-                    pos++;
+                    if (pos)
+                    {
+                        pos[0] = '\0';
+                        host = ptr_data + 1;
+                        pos++;
+                    }
+                    else
+                        pos = ptr_data;
                 }
                 else
                     pos = ptr_data;
@@ -859,11 +871,11 @@ server_recv (t_irc_server *server)
 }
 
 /*
- * server_kill_child: kill child process and close pipe
+ * server_child_kill: kill child process and close pipe
  */
 
 void
-server_kill_child (t_irc_server *server)
+server_child_kill (t_irc_server *server)
 {
     /* kill process */
     if (server->child_pid > 0)
@@ -893,7 +905,7 @@ server_kill_child (t_irc_server *server)
 void
 server_close_connection (t_irc_server *server)
 {
-    server_kill_child (server);
+    server_child_kill (server);
     
     /* close network socket */
     if (server->sock != -1)
@@ -951,7 +963,7 @@ server_child_read (t_irc_server *server)
     int num_read;
     
     num_read = read (server->child_read, buffer, sizeof (buffer));
-    if (num_read != -1)
+    if (num_read > 0)
     {
         switch (buffer[0])
         {
@@ -976,7 +988,7 @@ server_child_read (t_irc_server *server)
                 }
 #endif
                 /* kill child and login to server */
-                server_kill_child (server);
+                server_child_kill (server);
                 irc_login (server);
                 break;
             /* adress not found */
@@ -1662,7 +1674,7 @@ server_connect (t_irc_server *server)
             _exit (EXIT_SUCCESS);
     }
     
-    /* parent process go on here */
+    /* parent process */
     server->child_pid = pid;
     
     return 1;
@@ -1741,8 +1753,8 @@ server_disconnect (t_irc_server *server, int reconnect)
     
     if (server->nick_modes)
     {
-        memset (server->nick_modes, ' ', NUM_NICK_MODES);
-        server->nick_modes[NUM_NICK_MODES] = '\0';
+        free (server->nick_modes);
+        server->nick_modes = NULL;
     }
     server->is_away = 0;
     server->away_time = 0;
@@ -1923,6 +1935,53 @@ server_set_away (t_irc_server *server, char *nick, int is_away)
     }
 }
 
+/*
+ * server_get_default_notify_level: get default notify level for server
+ */
+
+int
+server_get_default_notify_level (t_irc_server *server)
+{
+    int notify, value;
+    char *pos;
+
+    notify = NOTIFY_LEVEL_DEFAULT;
+    
+    if (!server || !server->notify_levels)
+        return notify;
+    
+    pos = strstr (server->notify_levels, "*:");
+    if (pos)
+    {
+        pos += 2;
+        if (pos[0])
+        {
+            value = (int)(pos[0] - '0');
+            if ((value >= NOTIFY_LEVEL_MIN) && (value <= NOTIFY_LEVEL_MAX))
+                notify = value;
+        }
+    }
+
+    return notify;
+}
+
+/*
+ * server_set_default_notify_level: set default notify level for server
+ */
+
+void
+server_set_default_notify_level (t_irc_server *server, int notify)
+{
+    char level_string[2];
+
+    if (server)
+    {
+        level_string[0] = notify + '0';
+        level_string[1] = '\0';
+        config_option_list_set (&(server->notify_levels), "*", level_string);
+    }
+}
+
 /*
  * server_print_log: print server infos in log (usually for crash dump)
  */
@@ -1969,6 +2028,7 @@ server_print_log (t_irc_server *server)
     weechat_log_printf ("  reconnect_start . . : %ld\n",  server->reconnect_start);
     weechat_log_printf ("  reconnect_join. . . : %d\n",   server->reconnect_join);
     weechat_log_printf ("  is_away . . . . . . : %d\n",   server->is_away);
+    weechat_log_printf ("  away_message. . . . : '%s'\n", server->away_message);
     weechat_log_printf ("  away_time . . . . . : %ld\n",  server->away_time);
     weechat_log_printf ("  lag . . . . . . . . : %d\n",   server->lag);
     weechat_log_printf ("  lag_check_time. . . : tv_sec:%d, tv_usec:%d\n",
