@@ -85,21 +85,6 @@ struct t_irc_nick
 #define CHANNEL_TYPE_PRIVATE  1
 #define CHANNEL_TYPE_DCC_CHAT 2
 
-#define NUM_CHANNEL_MODES       8
-#define CHANNEL_MODE_INVITE     0
-#define CHANNEL_MODE_KEY        1
-#define CHANNEL_MODE_LIMIT      2
-#define CHANNEL_MODE_MODERATED  3
-#define CHANNEL_MODE_NO_MSG_OUT 4
-#define CHANNEL_MODE_SECRET     5
-#define CHANNEL_MODE_TOPIC      6
-#define CHANNEL_MODE_PRIVATE    7
-#define CHANNEL_SET_MODE(channel, set, mode) \
-    if (set) \
-        channel->modes[mode] = channel_modes[mode]; \
-    else \
-        channel->modes[mode] = ' ';
-
 typedef struct t_irc_channel t_irc_channel;
 
 struct t_irc_channel
@@ -116,6 +101,7 @@ struct t_irc_channel
     char *away_message;             /* to display away only once in private   */
     int cycle;                      /* currently cycling (/part then /join)   */
     int close;                      /* close request (/buffer close)          */
+    int display_creation_date;      /* 1 if creation date should be displayed */
     t_irc_nick *nicks;              /* nicks on the channel                   */
     t_irc_nick *last_nick;          /* last nick on the channel               */
     t_gui_buffer *buffer;           /* GUI buffer allocated for channel       */
@@ -125,20 +111,6 @@ struct t_irc_channel
 
 /* server types */
 
-#define NUM_NICK_MODES           7
-#define NICK_MODE_AWAY           0
-#define NICK_MODE_INVISIBLE      1
-#define NICK_MODE_WALLOPS        2
-#define NICK_MODE_RESTRICTED     3
-#define NICK_MODE_OPERATOR       4
-#define NICK_MODE_LOCAL_OPERATOR 5
-#define NICK_MODE_SERVER_NOTICES 6
-#define NICK_SET_MODE(server, set, mode) \
-    if (set) \
-        server->nick_modes[mode] = nick_modes[mode]; \
-    else \
-        server->nick_modes[mode] = ' ';
-
 typedef struct t_irc_server t_irc_server;
 
 struct t_irc_server
@@ -184,7 +156,8 @@ struct t_irc_server
     char *nick_modes;               /* nick modes                             */
     time_t reconnect_start;         /* this time + delay = reconnect time     */
     int reconnect_join;             /* 1 if channels opened to rejoin         */
-    int is_away;                    /* 1 is user is marker as away            */
+    int is_away;                    /* 1 is user is marked as away            */
+    char *away_message;             /* away message, NULL if not away         */
     time_t away_time;               /* time() when user marking as away       */
     int lag;                        /* lag (in milliseconds)                  */
     struct timeval lag_check_time;  /* last time lag was checked (ping sent)  */
@@ -200,6 +173,8 @@ struct t_irc_server
 
 /* irc commands */
 
+typedef int (t_irc_recv_func)(t_irc_server *, char *, char *, char *);
+
 typedef struct t_irc_command t_irc_command;
 
 struct t_irc_command
@@ -211,15 +186,14 @@ struct t_irc_command
     char *completion_template;      /* template for completion                */
                                     /* NULL=no completion, ""=default (nick)  */
     int min_arg, max_arg;           /* min & max number of arguments          */
-    int charset_conversion;         /* = 1 if cmd args are converted before   */
-                                    /* they're sent to server                 */
+    int conversion;                 /* = 1 if cmd args are converted (charset */
+                                    /* and color) before sending to server    */
     int needs_connection;           /* = 1 if cmd needs server connection     */
     int (*cmd_function_args)(t_irc_server *, t_irc_channel *, int, char **);
                                     /* function called when user enters cmd   */
     int (*cmd_function_1arg)(t_irc_server *, t_irc_channel *, char *);
                                     /* function called when user enters cmd   */
-    int (*recv_function)(t_irc_server *, char *, char *, char *);
-                                    /* function called when cmd is received   */
+    t_irc_recv_func *recv_function; /* function called when cmd is received   */
 };
 
 /* irc messages */
@@ -235,23 +209,44 @@ struct t_irc_message
 
 /* DCC types */
 
-#define DCC_CHAT_RECV           0   /* receiving DCC chat                     */
-#define DCC_CHAT_SEND           1   /* sending DCC chat                       */
-#define DCC_FILE_RECV           2   /* incoming DCC file                      */
-#define DCC_FILE_SEND           3   /* sending DCC file                       */
+#define DCC_CHAT_RECV            0  /* receiving DCC chat                     */
+#define DCC_CHAT_SEND            1  /* sending DCC chat                       */
+#define DCC_FILE_RECV            2  /* incoming DCC file                      */
+#define DCC_FILE_SEND            3  /* sending DCC file                       */
 
-#define DCC_WAITING             0   /* waiting for host answer                */
-#define DCC_CONNECTING          1   /* connecting to host                     */
-#define DCC_ACTIVE              2   /* sending/receiving data                 */
-#define DCC_DONE                3   /* transfer done                          */
-#define DCC_FAILED              4   /* DCC failed                             */
-#define DCC_ABORTED             5   /* DCC aborted by user                    */
+/* DCC status */
+
+#define DCC_WAITING              0  /* waiting for host answer                */
+#define DCC_CONNECTING           1  /* connecting to host                     */
+#define DCC_ACTIVE               2  /* sending/receiving data                 */
+#define DCC_DONE                 3  /* transfer done                          */
+#define DCC_FAILED               4  /* DCC failed                             */
+#define DCC_ABORTED              5  /* DCC aborted by user                    */
+
+/* DCC blocksize (for file) */
+
+#define DCC_MIN_BLOCKSIZE     1024  /* min DCC block size when sending file   */
+#define DCC_MAX_BLOCKSIZE   102400  /* max DCC block size when sending file   */
+
+/* DCC errors (for file) */
+
+#define DCC_NO_ERROR             0  /* used when no error to report, all ok!  */
+#define DCC_ERROR_READ_LOCAL     1  /* unable to read local file              */
+#define DCC_ERROR_SEND_BLOCK     2  /* unable to send block to receiver       */
+#define DCC_ERROR_READ_ACK       3  /* unable to read ACK from receiver       */
+#define DCC_ERROR_CONNECT_SENDER 4  /* unable to connect to sender            */
+#define DCC_ERROR_RECV_BLOCK     5  /* unable to receive block from sender    */
+#define DCC_ERROR_WRITE_LOCAL    6  /* unable to write to local file          */
+
+/* DCC macros for type */
 
 #define DCC_IS_CHAT(type) ((type == DCC_CHAT_RECV) || (type == DCC_CHAT_SEND))
 #define DCC_IS_FILE(type) ((type == DCC_FILE_RECV) || (type == DCC_FILE_SEND))
 #define DCC_IS_RECV(type) ((type == DCC_CHAT_RECV) || (type == DCC_FILE_RECV))
 #define DCC_IS_SEND(type) ((type == DCC_CHAT_SEND) || (type == DCC_FILE_SEND))
 
+/* DCC macro for status */
+
 #define DCC_ENDED(status) ((status == DCC_DONE) || (status == DCC_FAILED) || \
                           (status == DCC_ABORTED))
 
@@ -261,7 +256,7 @@ struct t_irc_dcc
 {
     t_irc_server *server;           /* irc server                             */
     t_irc_channel *channel;         /* irc channel (for DCC chat only)        */
-    int type;                       /* DCC type (send or receive)             */
+    int type;                       /* DCC type (file/chat, send/receive)     */
     int status;                     /* DCC status (waiting, sending, ..)      */
     time_t start_time;              /* the time when DCC started              */
     time_t start_transfer;          /* the time when DCC transfer started     */
@@ -269,11 +264,16 @@ struct t_irc_dcc
     int port;                       /* port                                   */
     char *nick;                     /* remote nick                            */
     int sock;                       /* socket for connection                  */
+    pid_t child_pid;                /* pid of child process (sending/recving) */
+    int child_read;                 /* to read into child pipe                */
+    int child_write;                /* to write into child pipe               */
     char *unterminated_message;     /* beginning of a message in input buf    */
+    int fast_send;                  /* fase send for files: does not wait ACK */
     int file;                       /* local file (for reading or writing)    */
     char *filename;                 /* filename (given by sender)             */
     char *local_filename;           /* local filename (with path)             */
     int filename_suffix;            /* suffix (.1 for ex) if renaming file    */
+    int blocksize;                  /* block size for sending file            */
     unsigned long size;             /* file size                              */
     unsigned long pos;              /* number of bytes received/sent          */
     unsigned long ack;              /* number of bytes received OK            */
@@ -316,6 +316,7 @@ extern t_irc_server *irc_servers;
 extern t_irc_message *recv_msgq, *msgq_last_msg;
 extern int check_away;
 extern t_irc_dcc *dcc_list;
+extern t_irc_dcc *last_dcc;
 extern char *dcc_status_string[6];
 extern char *channel_modes;
 extern char *nick_modes;
@@ -354,6 +355,8 @@ extern int server_name_already_exists (char *);
 extern void server_remove_away ();
 extern void server_check_away ();
 extern void server_set_away (t_irc_server *, char *, int);
+extern int server_get_default_notify_level (t_irc_server *);
+extern void server_set_default_notify_level (t_irc_server *, int);
 extern void server_print_log (t_irc_server *);
 
 /* proxy functions (irc-server.c) */
@@ -403,6 +406,11 @@ extern int nick_get_max_length (t_irc_channel *);
 extern void nick_set_away (t_irc_channel *, t_irc_nick *, int);
 extern void nick_print_log (t_irc_nick *);
 
+/* mode functions (irc-mode.c) */
+
+extern void irc_mode_channel_set (t_irc_channel *, char *);
+extern void irc_mode_user_set (t_irc_server *, char *);
+
 /* DCC functions (irc-dcc.c) */
 
 extern void dcc_redraw (int);
@@ -417,9 +425,11 @@ extern t_irc_dcc *dcc_add (t_irc_server *, int, unsigned long, int, char *, int,
                            char *, char *, unsigned long);
 extern void dcc_send_request (t_irc_server *, int, char *, char *);
 extern void dcc_chat_sendf (t_irc_dcc *, char *, ...);
+extern void dcc_file_send_fork (t_irc_dcc *);
+extern void dcc_file_recv_fork (t_irc_dcc *);
 extern void dcc_handle ();
 extern void dcc_end ();
-extern void dcc_print_log (t_irc_dcc *);
+extern void dcc_print_log ();
 
 /* IRC display (irc-diplay.c) */
 
@@ -439,6 +449,7 @@ extern void irc_login (t_irc_server *);
 extern int irc_cmd_send_admin (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_ame (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_amsg (t_irc_server *, t_irc_channel *, char *);
+extern void irc_send_away (t_irc_server *, char *);
 extern int irc_cmd_send_away (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_ban (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_ctcp (t_irc_server *, t_irc_channel *, char *);
@@ -471,6 +482,7 @@ extern int irc_cmd_send_part (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_ping (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_pong (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_query (t_irc_server *, t_irc_channel *, char *);
+extern void irc_send_quit_server (t_irc_server *, char *);
 extern int irc_cmd_send_quit (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_quote (t_irc_server *, t_irc_channel *, char *);
 extern int irc_cmd_send_rehash (t_irc_server *, t_irc_channel *, char *);
@@ -515,24 +527,22 @@ extern int irc_cmd_recv_server_mode_reason (t_irc_server *, char *, char *, char
 extern int irc_cmd_recv_server_msg (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_server_reply (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_topic (t_irc_server *, char *, char *, char *);
+extern int irc_cmd_recv_wallops (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_001 (t_irc_server *, char *, char *, char *);
-extern int irc_cmd_recv_004 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_221 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_301 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_302 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_303 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_305 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_306 (t_irc_server *, char *, char *, char *);
-extern int irc_cmd_recv_307 (t_irc_server *, char *, char *, char *);
+extern int irc_cmd_recv_whois_nick_msg (t_irc_server *, char *, char *, char *);
+extern int irc_cmd_recv_310 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_311 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_312 (t_irc_server *, char *, char *, char *);
-extern int irc_cmd_recv_313 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_314 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_315 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_317 (t_irc_server *, char *, char *, char *);
-extern int irc_cmd_recv_318 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_319 (t_irc_server *, char *, char *, char *);
-extern int irc_cmd_recv_320 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_321 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_322 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_323 (t_irc_server *, char *, char *, char *);
@@ -541,6 +551,7 @@ extern int irc_cmd_recv_329 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_331 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_332 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_333 (t_irc_server *, char *, char *, char *);
+extern int irc_cmd_recv_338 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_341 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_344 (t_irc_server *, char *, char *, char *);
 extern int irc_cmd_recv_345 (t_irc_server *, char *, char *, char *);
