@@ -38,6 +38,7 @@
 #include "plugins.h"
 #include "plugins-config.h"
 #include "../common/command.h"
+#include "../common/util.h"
 #include "../common/weeconfig.h"
 #include "../irc/irc.h"
 #include "../gui/gui.h"
@@ -46,6 +47,24 @@
 t_weechat_plugin *weechat_plugins = NULL;
 t_weechat_plugin *last_weechat_plugin = NULL;
 
+t_plugin_irc_color plugins_irc_colors[GUI_NUM_IRC_COLORS] =
+{ { /*  0 */ WEECHAT_IRC_COLOR_WHITE,        "white"        },
+  { /*  1 */ WEECHAT_IRC_COLOR_BLACK,        "black"        },
+  { /*  2 */ WEECHAT_IRC_COLOR_BLUE,         "blue"         },
+  { /*  3 */ WEECHAT_IRC_COLOR_GREEN,        "green"        },
+  { /*  4 */ WEECHAT_IRC_COLOR_LIGHTRED,     "lightred"     },
+  { /*  5 */ WEECHAT_IRC_COLOR_RED,          "red"          },
+  { /*  6 */ WEECHAT_IRC_COLOR_MAGENTA,      "magenta"      },
+  { /*  7 */ WEECHAT_IRC_COLOR_BROWN,        "brown"        },
+  { /*  8 */ WEECHAT_IRC_COLOR_YELLOW,       "yellow"       },
+  { /*  9 */ WEECHAT_IRC_COLOR_LIGHTGREEN,   "lightgreen"   },
+  { /* 10 */ WEECHAT_IRC_COLOR_CYAN,         "cyan"         },
+  { /* 11 */ WEECHAT_IRC_COLOR_LIGHTCYAN,    "lightcyan"    },
+  { /* 12 */ WEECHAT_IRC_COLOR_LIGHTBLUE,    "lightblue"    },
+  { /* 13 */ WEECHAT_IRC_COLOR_LIGHTMAGENTA, "lightmagenta" },
+  { /* 14 */ WEECHAT_IRC_COLOR_GRAY,         "gray"         },
+  { /* 15 */ WEECHAT_IRC_COLOR_LIGHTGRAY,    "lightgray"    }};
+
 
 /*
  * plugin_find_server_channel: find server/channel for command execution
@@ -282,7 +301,7 @@ plugin_cmd_handler_add (t_weechat_plugin *plugin, char *command,
         new_handler->description = (description) ? strdup (description) : NULL;
         new_handler->arguments = (arguments) ? strdup (arguments) : NULL;
         new_handler->arguments_description = (arguments_description) ? strdup (arguments_description) : NULL;
-        new_handler->completion_template = (completion_template) ? strdup (completion_template) : NULL;
+        new_handler->completion_template = (completion_template) ? strdup (completion_template) : strdup ("");
         new_handler->interval = 0;
         new_handler->remaining = 0;
         new_handler->handler = handler_func;
@@ -685,10 +704,7 @@ plugin_search_full_name (char *filename)
     name_with_ext = (char *)malloc (length);
     if (!name_with_ext)
         return strdup (filename);
-    name_with_ext[0] = '\0';
-    if (ascii_strncasecmp (filename, "lib", 3) != 0)
-        strcat (name_with_ext, "lib");
-    strcat (name_with_ext, filename);
+    strcpy (name_with_ext, filename);
     if (!strchr (filename, '.')
         && cfg_plugins_extension && cfg_plugins_extension[0])
         strcat (name_with_ext, cfg_plugins_extension);
@@ -861,6 +877,13 @@ plugin_load (char *filename)
 	new_plugin->get_nick_info = &weechat_plugin_get_nick_info;
 	new_plugin->free_nick_info = &weechat_plugin_free_nick_info;
         new_plugin->input_color = &weechat_plugin_input_color;
+        new_plugin->get_irc_color = &weechat_plugin_get_irc_color;
+        new_plugin->get_window_info = &weechat_plugin_get_window_info;
+        new_plugin->free_window_info = &weechat_plugin_free_window_info;
+        new_plugin->get_buffer_info = &weechat_plugin_get_buffer_info;
+        new_plugin->free_buffer_info = &weechat_plugin_free_buffer_info;
+        new_plugin->get_buffer_data = &weechat_plugin_get_buffer_data;
+        new_plugin->free_buffer_data = &weechat_plugin_free_buffer_data;
         
         /* handlers */
         new_plugin->handlers = NULL;
