@@ -46,11 +46,11 @@ extern void boot_DynaLoader (pTHX_ CV* cv);
 
 #ifndef MULTIPLICITY
 #define PKG_NAME_PREFIX "WeechatPerlPackage"
-static PerlInterpreter *main_perl = NULL;
-int packnum = 0;
+static PerlInterpreter *perl_main = NULL;
+int perl_num = 0;
 #endif
 
-char *weechat_perl_code =
+char *perl_weechat_code =
 {
 #ifndef MULTIPLICITY
     "package WeechatPerlScriptLoader;"
@@ -241,12 +241,13 @@ weechat_perl_keyboard_handler (t_weechat_plugin *plugin,
 static XS (XS_weechat_register)
 {
     char *name, *version, *shutdown_func, *description;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
     (void) items;
     (void) cv;
+
+    perl_current_script = NULL;
     
     if (items != 4)
     {
@@ -256,10 +257,10 @@ static XS (XS_weechat_register)
         XSRETURN_NO;
     }
     
-    name = SvPV (ST (0), integer);
-    version = SvPV (ST (1), integer);
-    shutdown_func = SvPV (ST (2), integer);
-    description = SvPV (ST (3), integer);
+    name = SvPV (ST (0), PL_na);
+    version = SvPV (ST (1), PL_na);
+    shutdown_func = SvPV (ST (2), PL_na);
+    description = SvPV (ST (3), PL_na);
     
     if (weechat_script_search (perl_plugin, &perl_scripts, name))
     {
@@ -288,10 +289,6 @@ static XS (XS_weechat_register)
     }
     else
     {
-        perl_plugin->print_server (perl_plugin,
-                                   "Perl error: unable to load script "
-                                   "\"%s\" (not enough memory)",
-                                   name);
         XSRETURN_NO;
     }
     
@@ -304,7 +301,6 @@ static XS (XS_weechat_register)
 
 static XS (XS_weechat_print)
 {
-    unsigned int integer;
     char *message, *channel_name, *server_name;
     dXSARGS;
     
@@ -327,16 +323,16 @@ static XS (XS_weechat_print)
         XSRETURN_NO;
     }
     
-    message = SvPV (ST (0), integer);
+    message = SvPV (ST (0), PL_na);
     
     channel_name = NULL;
     server_name = NULL;
     
     if (items > 1)
     {
-        channel_name = SvPV (ST (1), integer);
+        channel_name = SvPV (ST (1), PL_na);
         if (items > 2)
-            server_name = SvPV (ST (2), integer);
+            server_name = SvPV (ST (2), PL_na);
     }
     
     perl_plugin->print (perl_plugin,
@@ -346,13 +342,47 @@ static XS (XS_weechat_print)
     XSRETURN_YES;
 }
 
+/*
+ * weechat::print_server: print message into a server buffer
+ */
+
+static XS (XS_weechat_print_server)
+{
+    char *message;
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to print message, "
+                                   "script not initialized");
+	XSRETURN_NO;
+    }
+
+    if (items < 1)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"print_server\" function");
+        XSRETURN_NO;
+    }
+    
+    message = SvPV (ST (0), PL_na);
+    
+    perl_plugin->print_server (perl_plugin, "%s", message);
+    
+    XSRETURN_YES;
+}
+
 /*
  * weechat::print_infobar: print message to infobar
  */
 
 static XS (XS_weechat_print_infobar)
 {
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -377,7 +407,7 @@ static XS (XS_weechat_print_infobar)
     perl_plugin->print_infobar (perl_plugin,
                                 SvIV (ST (0)),
                                 "%s",
-                                SvPV (ST (1), integer));
+                                SvPV (ST (1), PL_na));
     
     XSRETURN_YES;
 }
@@ -413,7 +443,6 @@ static XS (XS_weechat_remove_infobar)
 
 static XS (XS_weechat_log)
 {
-    unsigned int integer;
     char *message, *channel_name, *server_name;
     dXSARGS;
     
@@ -436,16 +465,16 @@ static XS (XS_weechat_log)
         XSRETURN_NO;
     }
     
-    message = SvPV (ST (0), integer);
+    message = SvPV (ST (0), PL_na);
     
     channel_name = NULL;
     server_name = NULL;
     
     if (items > 1)
     {
-        channel_name = SvPV (ST (1), integer);
+        channel_name = SvPV (ST (1), PL_na);
         if (items > 2)
-            server_name = SvPV (ST (2), integer);
+            server_name = SvPV (ST (2), PL_na);
     }
     
     perl_plugin->log (perl_plugin,
@@ -461,7 +490,6 @@ static XS (XS_weechat_log)
 
 static XS (XS_weechat_command)
 {
-    unsigned int integer;
     char *channel_name, *server_name;
     dXSARGS;
     
@@ -489,14 +517,14 @@ static XS (XS_weechat_command)
     
     if (items > 1)
     {
-        channel_name = SvPV (ST (1), integer);
+        channel_name = SvPV (ST (1), PL_na);
         if (items > 2)
-            server_name = SvPV (ST (2), integer);
+            server_name = SvPV (ST (2), PL_na);
     }
     
     perl_plugin->exec_command (perl_plugin,
                                server_name, channel_name,
-                               SvPV (ST (0), integer));
+                               SvPV (ST (0), PL_na));
     
     XSRETURN_YES;
 }
@@ -508,7 +536,6 @@ static XS (XS_weechat_command)
 static XS (XS_weechat_add_message_handler)
 {
     char *irc_command, *function;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -530,8 +557,8 @@ static XS (XS_weechat_add_message_handler)
         XSRETURN_NO;
     }
     
-    irc_command = SvPV (ST (0), integer);
-    function = SvPV (ST (1), integer);
+    irc_command = SvPV (ST (0), PL_na);
+    function = SvPV (ST (1), PL_na);
     
     if (perl_plugin->msg_handler_add (perl_plugin, irc_command,
                                       weechat_perl_cmd_msg_handler, function,
@@ -549,7 +576,6 @@ static XS (XS_weechat_add_command_handler)
 {
     char *command, *function, *description, *arguments, *arguments_description;
     char *completion_template;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -571,12 +597,12 @@ static XS (XS_weechat_add_command_handler)
         XSRETURN_NO;
     }
     
-    command = SvPV (ST (0), integer);
-    function = SvPV (ST (1), integer);
-    description = (items >= 3) ? SvPV (ST (2), integer) : NULL;
-    arguments = (items >= 4) ? SvPV (ST (3), integer) : NULL;
-    arguments_description = (items >= 5) ? SvPV (ST (4), integer) : NULL;
-    completion_template = (items >= 6) ? SvPV (ST (5), integer) : NULL;
+    command = SvPV (ST (0), PL_na);
+    function = SvPV (ST (1), PL_na);
+    description = (items >= 3) ? SvPV (ST (2), PL_na) : NULL;
+    arguments = (items >= 4) ? SvPV (ST (3), PL_na) : NULL;
+    arguments_description = (items >= 5) ? SvPV (ST (4), PL_na) : NULL;
+    completion_template = (items >= 6) ? SvPV (ST (5), PL_na) : NULL;
     
     if (perl_plugin->cmd_handler_add (perl_plugin,
                                       command,
@@ -600,7 +626,6 @@ static XS (XS_weechat_add_timer_handler)
 {
     int interval;
     char *function;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -623,7 +648,7 @@ static XS (XS_weechat_add_timer_handler)
     }
     
     interval = SvIV (ST (0));
-    function = SvPV (ST (1), integer);
+    function = SvPV (ST (1), PL_na);
     
     if (perl_plugin->timer_handler_add (perl_plugin, interval,
                                         weechat_perl_timer_handler, function,
@@ -640,7 +665,6 @@ static XS (XS_weechat_add_timer_handler)
 static XS (XS_weechat_add_keyboard_handler)
 {
     char *function;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -662,7 +686,7 @@ static XS (XS_weechat_add_keyboard_handler)
         XSRETURN_NO;
     }
     
-    function = SvPV (ST (0), integer);
+    function = SvPV (ST (0), PL_na);
     
     if (perl_plugin->keyboard_handler_add (perl_plugin,
                                            weechat_perl_keyboard_handler,
@@ -680,7 +704,6 @@ static XS (XS_weechat_add_keyboard_handler)
 static XS (XS_weechat_remove_handler)
 {
     char *command, *function;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -702,8 +725,8 @@ static XS (XS_weechat_remove_handler)
         XSRETURN_NO;
     }
     
-    command = SvPV (ST (0), integer);
-    function = SvPV (ST (1), integer);
+    command = SvPV (ST (0), PL_na);
+    function = SvPV (ST (1), PL_na);
     
     weechat_script_remove_handler (perl_plugin, perl_current_script,
                                    command, function);
@@ -718,7 +741,6 @@ static XS (XS_weechat_remove_handler)
 static XS (XS_weechat_remove_timer_handler)
 {
     char *function;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -740,7 +762,7 @@ static XS (XS_weechat_remove_timer_handler)
         XSRETURN_NO;
     }
     
-    function = SvPV (ST (0), integer);
+    function = SvPV (ST (0), PL_na);
     
     weechat_script_remove_timer_handler (perl_plugin, perl_current_script,
                                          function);
@@ -755,7 +777,6 @@ static XS (XS_weechat_remove_timer_handler)
 static XS (XS_weechat_remove_keyboard_handler)
 {
     char *function;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -777,7 +798,7 @@ static XS (XS_weechat_remove_keyboard_handler)
         XSRETURN_NO;
     }
     
-    function = SvPV (ST (0), integer);
+    function = SvPV (ST (0), PL_na);
     
     weechat_script_remove_keyboard_handler (perl_plugin, perl_current_script,
                                             function);
@@ -792,7 +813,6 @@ static XS (XS_weechat_remove_keyboard_handler)
 static XS (XS_weechat_get_info)
 {
     char *arg, *info, *server_name;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -814,9 +834,9 @@ static XS (XS_weechat_get_info)
         XSRETURN_NO;
     }
     
-    server_name = (items == 2) ? SvPV (ST (1), integer) : NULL;
+    server_name = (items == 2) ? SvPV (ST (1), PL_na) : NULL;
     
-    arg = SvPV (ST (0), integer);
+    arg = SvPV (ST (0), PL_na);
     if (arg)
     {
         info = perl_plugin->get_info (perl_plugin, arg, server_name);
@@ -906,7 +926,6 @@ static XS (XS_weechat_get_dcc_info)
 static XS (XS_weechat_get_config)
 {
     char *option, *return_value;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -917,7 +936,7 @@ static XS (XS_weechat_get_config)
         perl_plugin->print_server (perl_plugin,
                                    "Perl error: unable to get config option, "
                                    "script not initialized");
-	XSRETURN_NO;
+	XSRETURN_EMPTY;
     }
     
     if (items < 1)
@@ -925,10 +944,10 @@ static XS (XS_weechat_get_config)
         perl_plugin->print_server (perl_plugin,
                                    "Perl error: wrong parameters for "
                                    "\"get_config\" function");
-        XSRETURN_NO;
+        XSRETURN_EMPTY;
     }
     
-    option = SvPV (ST (0), integer);
+    option = SvPV (ST (0), PL_na);
     
     if (option)
     {
@@ -953,7 +972,6 @@ static XS (XS_weechat_get_config)
 static XS (XS_weechat_set_config)
 {
     char *option, *value;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -975,8 +993,8 @@ static XS (XS_weechat_set_config)
         XSRETURN_NO;
     }
     
-    option = SvPV (ST (0), integer);
-    value = SvPV (ST (1), integer);
+    option = SvPV (ST (0), PL_na);
+    value = SvPV (ST (1), PL_na);
     
     if (option && value)
     {
@@ -994,7 +1012,6 @@ static XS (XS_weechat_set_config)
 static XS (XS_weechat_get_plugin_config)
 {
     char *option, *return_value;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -1005,7 +1022,7 @@ static XS (XS_weechat_get_plugin_config)
         perl_plugin->print_server (perl_plugin,
                                    "Perl error: unable to get plugin config option, "
                                    "script not initialized");
-	XSRETURN_NO;
+	XSRETURN_EMPTY;
     }
     
     if (items < 1)
@@ -1013,10 +1030,10 @@ static XS (XS_weechat_get_plugin_config)
         perl_plugin->print_server (perl_plugin,
                                    "Perl error: wrong parameters for "
                                    "\"get_plugin_config\" function");
-        XSRETURN_NO;
+        XSRETURN_EMPTY;
     }
     
-    option = SvPV (ST (0), integer);
+    option = SvPV (ST (0), PL_na);
     
     if (option)
     {
@@ -1043,7 +1060,6 @@ static XS (XS_weechat_get_plugin_config)
 static XS (XS_weechat_set_plugin_config)
 {
     char *option, *value;
-    unsigned int integer;
     dXSARGS;
     
     /* make gcc happy */
@@ -1065,8 +1081,8 @@ static XS (XS_weechat_set_plugin_config)
         XSRETURN_NO;
     }
     
-    option = SvPV (ST (0), integer);
-    value = SvPV (ST (1), integer);
+    option = SvPV (ST (0), PL_na);
+    value = SvPV (ST (1), PL_na);
     
     if (option && value)
     {
@@ -1147,6 +1163,7 @@ static XS (XS_weechat_get_server_info)
         hv_store (server_hash_member, "is_connected",         12, newSViv (ptr_server->is_connected), 0);
         hv_store (server_hash_member, "ssl_connected",        13, newSViv (ptr_server->ssl_connected), 0);
         hv_store (server_hash_member, "nick",                  4, newSVpv (ptr_server->nick, 0), 0);
+        hv_store (server_hash_member, "nick_modes",           10, newSVpv (ptr_server->nick_modes, 0), 0);
 	hv_store (server_hash_member, "away_time",             9, newSVpv (timebuffer, 0), 0);
 	hv_store (server_hash_member, "lag",                   3, newSViv (ptr_server->lag), 0);
 	
@@ -1154,7 +1171,6 @@ static XS (XS_weechat_get_server_info)
     }    
     perl_plugin->free_server_info (perl_plugin, server_info);
     
-    //XPUSHs(newRV_inc((SV *) server_hash));    
     ST (0) = newRV_inc((SV *) server_hash);
     XSRETURN (1);
 }
@@ -1167,7 +1183,6 @@ static XS (XS_weechat_get_channel_info)
 {
     t_plugin_channel_info *channel_info, *ptr_channel;
     char *server;
-    unsigned int integer;
     HV *channel_hash, *channel_hash_member;
     dXSARGS;
     
@@ -1190,7 +1205,7 @@ static XS (XS_weechat_get_channel_info)
         XSRETURN_EMPTY;
     }
     
-    server = SvPV (ST (0), integer);
+    server = SvPV (ST (0), PL_na);
     if (!server)
 	XSRETURN_EMPTY;
 
@@ -1222,7 +1237,6 @@ static XS (XS_weechat_get_channel_info)
     }
     perl_plugin->free_channel_info (perl_plugin, channel_info);
     
-    //XPUSHs(newRV_inc((SV *) channel_hash));
     ST (0) = newRV_inc((SV *) channel_hash);
     XSRETURN (1);
 }
@@ -1235,7 +1249,6 @@ static XS (XS_weechat_get_nick_info)
 {
     t_plugin_nick_info *nick_info, *ptr_nick;
     char *server, *channel;
-    unsigned int integer;
     HV *nick_hash;
     dXSARGS;
     
@@ -1258,8 +1271,8 @@ static XS (XS_weechat_get_nick_info)
         XSRETURN_EMPTY;
     }
     
-    server = SvPV (ST (0), integer);
-    channel = SvPV (ST (1), integer);
+    server = SvPV (ST (0), PL_na);
+    channel = SvPV (ST (1), PL_na);
     if (!server || !channel)
 	XSRETURN_EMPTY;
     
@@ -1288,7 +1301,6 @@ static XS (XS_weechat_get_nick_info)
     }
     perl_plugin->free_nick_info (perl_plugin, nick_info);
     
-    //XPUSHs(newRV_inc((SV *) nick_hash));
     ST (0) = newRV_inc((SV *) nick_hash);
     XSRETURN (1);
 }
@@ -1330,6 +1342,220 @@ static XS (XS_weechat_input_color)
     XSRETURN_YES;
 }
 
+/*
+ * weechat::get_irc_color:
+ *          get the numeric value which identify an irc color by its name
+ */
+
+static XS (XS_weechat_get_irc_color)
+{
+    char *color;
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to get irc color, "
+                                   "script not initialized");
+	XST_mIV (0, -1);
+	XSRETURN (1);
+    }
+    
+    if (items != 1)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"get_irc_info\" function");
+	XST_mIV (0, -1);
+	XSRETURN (1);
+    }
+    
+    color = SvPV (ST (0), PL_na);
+    if (color)
+    {
+	XST_mIV (0, perl_plugin->get_irc_color (perl_plugin, color));
+	XSRETURN (1);
+    }
+
+    XST_mIV (0, -1);
+    XSRETURN (-1);
+}
+
+/*
+ * weechat::get_window_info: get infos about windows
+ */
+
+static XS (XS_weechat_get_window_info)
+{
+    t_plugin_window_info *window_info, *ptr_window;
+    int count;
+    HV *window_hash_member;
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    (void) items;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to get window info, "
+                                   "script not initialized");
+	XSRETURN_EMPTY;
+    }
+    
+    window_info = perl_plugin->get_window_info (perl_plugin);
+    count = 0;
+    if (!window_info)
+	XSRETURN_EMPTY;
+    
+    for (ptr_window = window_info; ptr_window; ptr_window = ptr_window->next_window)
+    {
+	window_hash_member = (HV *) sv_2mortal((SV *) newHV());	        
+        
+	hv_store (window_hash_member, "num_buffer",     10, newSViv (ptr_window->num_buffer), 0);
+        hv_store (window_hash_member, "win_x",           5, newSViv (ptr_window->win_x), 0);
+        hv_store (window_hash_member, "win_y",           5, newSViv (ptr_window->win_y), 0);
+        hv_store (window_hash_member, "win_width",       9, newSViv (ptr_window->win_width), 0);
+	hv_store (window_hash_member, "win_height",     10, newSViv (ptr_window->win_height), 0);
+	hv_store (window_hash_member, "win_width_pct",  13, newSViv (ptr_window->win_width_pct), 0);
+	hv_store (window_hash_member, "win_height_pct", 14, newSViv (ptr_window->win_height_pct), 0);
+	
+	XPUSHs(newRV_inc((SV *) window_hash_member));
+	count++;
+    }    
+    perl_plugin->free_window_info (perl_plugin, window_info);
+    
+    XSRETURN (count);
+}
+
+/*
+ * weechat::get_buffer_info: get infos about buffers
+ */
+
+static XS (XS_weechat_get_buffer_info)
+{
+    t_plugin_buffer_info *buffer_info, *ptr_buffer;
+    HV *buffer_hash, *buffer_hash_member;
+    char conv[8];
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    (void) items;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to get buffer info, "
+                                   "script not initialized");
+	XSRETURN_EMPTY;
+    }
+    
+    buffer_info = perl_plugin->get_buffer_info (perl_plugin);
+    if (!buffer_info)
+    {
+	XSRETURN_EMPTY;
+    }    
+    
+    buffer_hash = (HV *) sv_2mortal((SV *) newHV());
+    if (!buffer_hash)
+    {
+        perl_plugin->free_buffer_info (perl_plugin, buffer_info);
+	XSRETURN_EMPTY;
+    }
+    
+    for (ptr_buffer = buffer_info; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
+    {
+	buffer_hash_member = (HV *) sv_2mortal((SV *) newHV());	        
+        
+	hv_store (buffer_hash_member, "type",           4, newSViv (ptr_buffer->type), 0);
+        hv_store (buffer_hash_member, "num_displayed", 13, newSViv (ptr_buffer->num_displayed), 0);
+	hv_store (buffer_hash_member, "server",         6, 
+		  newSVpv (ptr_buffer->server_name == NULL ? "" : ptr_buffer->server_name, 0), 0);
+        hv_store (buffer_hash_member, "channel",        7, 
+		  newSVpv (ptr_buffer->channel_name == NULL ? "" : ptr_buffer->channel_name, 0), 0);
+        hv_store (buffer_hash_member, "notify_level",  12, newSViv (ptr_buffer->notify_level), 0);
+	hv_store (buffer_hash_member, "log_filename",  12, 
+		  newSVpv (ptr_buffer->log_filename == NULL ? "" : ptr_buffer->log_filename, 0), 0);
+	
+	snprintf(conv, sizeof(conv), "%d", ptr_buffer->number);
+	hv_store (buffer_hash, conv, strlen(conv), newRV_inc((SV *) buffer_hash_member), 0);	
+    }    
+    perl_plugin->free_buffer_info (perl_plugin, buffer_info);
+    
+    ST (0) = newRV_inc((SV *) buffer_hash);
+    XSRETURN (1);
+}
+
+/*
+ * weechat::get_buffer_data: get buffer content
+ */
+
+static XS (XS_weechat_get_buffer_data)
+{
+    t_plugin_buffer_line *buffer_data, *ptr_data;
+    HV *data_list_member;
+    char *server, *channel;
+    int count;
+    
+    dXSARGS;
+    
+    /* make gcc happy */
+    (void) cv;
+    (void) items;
+    
+    if (!perl_current_script)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: unable to get buffer data, "
+                                   "script not initialized");
+	XSRETURN_EMPTY;
+    }
+    
+    if (items > 2)
+    {
+        perl_plugin->print_server (perl_plugin,
+                                   "Perl error: wrong parameters for "
+                                   "\"get_buffer_data\" function");
+        XSRETURN_EMPTY;
+    }
+    
+    channel = NULL;
+    server = NULL;
+
+    if (items >= 1)
+	server = SvPV (ST (0), PL_na);
+    if (items >= 2)
+	channel = SvPV (ST (1), PL_na);
+
+    SP -= items;
+    
+    buffer_data = perl_plugin->get_buffer_data (perl_plugin, server, channel);
+    count = 0;
+    if (!buffer_data)
+	XSRETURN_EMPTY;
+    
+    for (ptr_data = buffer_data; ptr_data; ptr_data = ptr_data->next_line)
+    {
+	data_list_member = (HV *) sv_2mortal((SV *) newHV());
+        
+	hv_store (data_list_member, "nick", 4, 
+		  newSVpv (ptr_data->nick == NULL ? "" : ptr_data->nick, 0), 0);
+	hv_store (data_list_member, "data", 4, 
+		  newSVpv (ptr_data->data == NULL ? "" : ptr_data->data, 0), 0);
+	
+	XPUSHs(newRV_inc((SV *) data_list_member));
+	count++;
+    }    
+    perl_plugin->free_buffer_data (perl_plugin, buffer_data);
+    
+    XSRETURN (count);
+}
+
 /*
  * weechat_perl_xs_init: initialize subroutines
  */
@@ -1344,6 +1570,7 @@ weechat_perl_xs_init (pTHX)
     /* interface functions */
     newXS ("weechat::register", XS_weechat_register, "weechat");
     newXS ("weechat::print", XS_weechat_print, "weechat");
+    newXS ("weechat::print_server", XS_weechat_print_server, "weechat");
     newXS ("weechat::print_infobar", XS_weechat_print_infobar, "weechat");
     newXS ("weechat::remove_infobar", XS_weechat_remove_infobar, "weechat");
     newXS ("weechat::log", XS_weechat_log, "weechat");
@@ -1365,7 +1592,11 @@ weechat_perl_xs_init (pTHX)
     newXS ("weechat::get_channel_info", XS_weechat_get_channel_info, "weechat");
     newXS ("weechat::get_nick_info", XS_weechat_get_nick_info, "weechat");
     newXS ("weechat::input_color", XS_weechat_input_color, "weechat");
-    
+    newXS ("weechat::get_irc_color", XS_weechat_get_irc_color, "weechat");
+    newXS ("weechat::get_window_info", XS_weechat_get_window_info, "weechat");
+    newXS ("weechat::get_buffer_info", XS_weechat_get_buffer_info, "weechat");
+    newXS ("weechat::get_buffer_data", XS_weechat_get_buffer_data, "weechat");
+
     /* interface constants */
     stash = gv_stashpv ("weechat", TRUE);
     newCONSTSUB (stash, "weechat::PLUGIN_RC_KO", newSViv (PLUGIN_RC_KO));
@@ -1397,8 +1628,8 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
     perl_current_script = NULL;
 
 #ifndef MULTIPLICITY
-    snprintf(pkgname, sizeof(pkgname), "%s%d", PKG_NAME_PREFIX, packnum);
-    packnum++;
+    snprintf(pkgname, sizeof(pkgname), "%s%d", PKG_NAME_PREFIX, perl_num);
+    perl_num++;
     tempscript.interpreter = "WeechatPerlScriptLoader";
     eval = weechat_perl_exec (plugin, &tempscript, "weechat_perl_load_eval_file", filename, pkgname, "");
 #else
@@ -1418,7 +1649,7 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
     tempscript.interpreter = (PerlInterpreter *) perl_current_interpreter;
     perl_parse (perl_current_interpreter, weechat_perl_xs_init, 3, perl_args, NULL);
     
-    eval_pv (weechat_perl_code, TRUE);
+    eval_pv (perl_weechat_code, TRUE);
     eval = weechat_perl_exec (plugin, &tempscript, "weechat_perl_load_eval_file", filename, "", "");
 
     free (perl_current_script_filename);
@@ -1465,7 +1696,7 @@ weechat_perl_load (t_weechat_plugin *plugin, char *filename)
     {
 	plugin->print_server (plugin,
                               "Perl error: function \"register\" not found "
-                              "in file \"%s\"",
+                              "(or failed) in file \"%s\"",
                               filename);
 #ifdef MULTIPLICITY
         perl_destruct (perl_current_interpreter);
@@ -1677,6 +1908,8 @@ weechat_perl_cmd (t_weechat_plugin *plugin,
                                           ptr_handler->handler_args);
                 }
             }
+            if (!handler_found)
+                plugin->print_server (plugin, "  (none)");
             break;
         case 1:
             if (plugin->ascii_strcasecmp (plugin, argv[0], "autoload") == 0)
@@ -1737,24 +1970,25 @@ weechat_plugin_init (t_weechat_plugin *plugin)
    
     plugin->print_server (plugin, "Loading Perl module \"weechat\" (without multiplicity)");
     
-    main_perl = perl_alloc ();
+    perl_main = perl_alloc ();
     
-    if (!main_perl)
+    if (!perl_main)
     {
         plugin->print_server (plugin,
                               "Perl error: unable to initialize Perl");
         return PLUGIN_RC_KO;
     }
     
-    perl_construct (main_perl);
-    perl_parse (main_perl, weechat_perl_xs_init, 3, perl_args, NULL);
-    eval_pv (weechat_perl_code, TRUE);
+    perl_construct (perl_main);
+    perl_parse (perl_main, weechat_perl_xs_init, 3, perl_args, NULL);
+    eval_pv (perl_weechat_code, TRUE);
 #endif
     
     plugin->cmd_handler_add (plugin, "perl",
                              "list/load/unload Perl scripts",
-                             "[load filename] | [autoload] | [reload] | [unload]",
-                             "filename: Perl script (file) to load\n\n"
+                             "[load filename] | [autoload] | [reload] | [unload [script]]",
+                             "filename: Perl script (file) to load\n"
+                             "script: script name to unload\n\n"
                              "Without argument, /perl command lists all loaded Perl scripts.",
                              "load|autoload|reload|unload",
                              weechat_perl_cmd, NULL, NULL);
@@ -1780,11 +2014,11 @@ weechat_plugin_end (t_weechat_plugin *plugin)
 
 #ifndef MULTIPLICITY
     /* free perl intepreter */
-    if (main_perl)
+    if (perl_main)
     {
-	perl_destruct (main_perl);
-	perl_free (main_perl);
-	main_perl = NULL;
+	perl_destruct (perl_main);
+	perl_free (perl_main);
+	perl_main = NULL;
     }
 #endif
     
