@@ -57,7 +57,8 @@ weechat_python_exec (t_weechat_plugin *plugin,
     PyObject *evFunc;
     PyObject *rc;
     int ret;
-
+    
+    /* PyEval_AcquireLock (); */
     PyThreadState_Swap (script->interpreter);
     
     evMain = PyImport_AddModule ((char *) "__main__");
@@ -69,7 +70,8 @@ weechat_python_exec (t_weechat_plugin *plugin,
         plugin->print_server (plugin,
                               "Python error: unable to run function \"%s\"",
                               function);
-        return PLUGIN_RC_KO;
+	/* PyEval_ReleaseThread (python_current_script->interpreter); */
+	return PLUGIN_RC_KO;
     }
 
     ret = -1;
@@ -95,6 +97,7 @@ weechat_python_exec (t_weechat_plugin *plugin,
     if (rc == Py_None)
     {
 	python_plugin->print_server (python_plugin, "Python error: function \"%s\" must return a valid value", function);
+	/* PyEval_ReleaseThread (python_current_script->interpreter); */
 	return PLUGIN_RC_OK;
     }
 
@@ -107,9 +110,11 @@ weechat_python_exec (t_weechat_plugin *plugin,
     if (PyErr_Occurred ()) PyErr_Print ();
     
     if (ret < 0)
-        return PLUGIN_RC_OK;
-    else
-        return ret;
+	ret = PLUGIN_RC_OK;
+    
+    /* PyEval_ReleaseThread (python_current_script->interpreter); */
+    
+    return ret;
 }
 
 /*
@@ -172,6 +177,8 @@ weechat_python_register (PyObject *self, PyObject *args)
     
     /* make gcc happy */
     (void) self;
+
+    python_current_script = NULL;
     
     name = NULL;
     version = NULL;
@@ -212,13 +219,7 @@ weechat_python_register (PyObject *self, PyObject *args)
                                      name, version, description);
     }
     else
-    {
-        python_plugin->print_server (python_plugin,
-                                     "Python error: unable to load script "
-                                     "\"%s\" (not enough memory)",
-                                     name);
         return Py_BuildValue ("i", 0);
-    }
     
     return Py_BuildValue ("i", 1);
 }
@@ -262,6 +263,41 @@ weechat_python_print (PyObject *self, PyObject *args)
     return Py_BuildValue ("i", 1);
 }
 
+/*
+ * weechat_python_print_server: print message into a buffer server
+ */
+
+static PyObject *
+weechat_python_print_server (PyObject *self, PyObject *args)
+{
+    char *message;
+    
+    /* make gcc happy */
+    (void) self;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to print message (server), "
+                                     "script not initialized");
+        return Py_BuildValue ("i", 0);
+    }
+    
+    message = NULL;
+    
+    if (!PyArg_ParseTuple (args, "s", &message))
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: wrong parameters for "
+                                     "\"print_server\" function");
+        return Py_BuildValue ("i", 0);
+    }
+    
+    python_plugin->print_server (python_plugin, "%s", message);
+    
+    return Py_BuildValue ("i", 1);
+}
+
 /*
  * weechat_python_print_infobar: print message to infobar
  */
@@ -817,22 +853,9 @@ weechat_python_get_dcc_info (PyObject *self, PyObject *args)
 	    PyDict_SetItem(dcc_list_member, Py_BuildValue("s", "start_resume"),
 			   Py_BuildValue("k", ptr_dcc->start_resume));
 	    PyDict_SetItem(dcc_list_member, Py_BuildValue("s", "cps"),
-			   Py_BuildValue("k", ptr_dcc->bytes_per_sec));
+			   Py_BuildValue("k", ptr_dcc->bytes_per_sec));	    
 	    
-	    
-            if (PyList_Append(dcc_list, dcc_list_member) != 0)
-            {
-		Py_DECREF(dcc_list_member);
-		Py_DECREF(dcc_list);
-                python_plugin->free_dcc_info (python_plugin, dcc_info);
-                return Py_None;
-            }
-        }
-        else
-        {
-	    Py_DECREF(dcc_list);
-            python_plugin->free_dcc_info (python_plugin, dcc_info);
-            return Py_None;
+            PyList_Append(dcc_list, dcc_list_member);
         }
     }
     
@@ -859,7 +882,7 @@ weechat_python_get_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get config option, "
                                      "script not initialized");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     option = NULL;
@@ -869,7 +892,7 @@ weechat_python_get_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_config\" function");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     if (option)
@@ -945,7 +968,7 @@ weechat_python_get_plugin_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get plugin config option, "
                                      "script not initialized");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     option = NULL;
@@ -955,7 +978,7 @@ weechat_python_get_plugin_config (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_plugin_config\" function");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     if (option)
@@ -1037,7 +1060,7 @@ weechat_python_get_server_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get server infos, "
                                      "script not initialized");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     server_hash = PyDict_New ();
@@ -1107,6 +1130,8 @@ weechat_python_get_server_info (PyObject *self, PyObject *args)
 			   Py_BuildValue("i", ptr_server->ssl_connected));
 	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "nick"),
 			   Py_BuildValue("s", ptr_server->nick));
+            PyDict_SetItem(server_hash_member, Py_BuildValue("s", "nick_modes"),
+			   Py_BuildValue("s", ptr_server->nick_modes));
 	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "away_time"),
 			   Py_BuildValue("s", timebuffer));
 	    PyDict_SetItem(server_hash_member, Py_BuildValue("s", "lag"),
@@ -1140,7 +1165,7 @@ weechat_python_get_channel_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get channel infos, "
                                      "script not initialized");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     server = NULL;
@@ -1149,7 +1174,7 @@ weechat_python_get_channel_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_channel_info\" function");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     channel_hash = PyDict_New ();
@@ -1207,7 +1232,7 @@ weechat_python_get_nick_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: unable to get nick infos, "
                                      "script not initialized");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     server = NULL;
@@ -1217,7 +1242,7 @@ weechat_python_get_nick_info (PyObject *self, PyObject *args)
         python_plugin->print_server (python_plugin,
                                      "Python error: wrong parameters for "
                                      "\"get_nick_info\" function");
-        return Py_BuildValue ("i", 0);
+        return Py_None;
     }
     
     nick_hash = PyDict_New ();
@@ -1248,6 +1273,221 @@ weechat_python_get_nick_info (PyObject *self, PyObject *args)
     return nick_hash;
 }
 
+/*
+ * weechat_python_get_irc_color: 
+ *          get the numeric value which identify an irc color by its name
+ */
+
+static PyObject *
+weechat_python_get_irc_color (PyObject *self, PyObject *args)
+{
+    char *color;
+    
+    /* make gcc happy */
+    (void) self;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to get irc color, "
+                                     "script not initialized");
+        return Py_BuildValue ("i", -1);
+    }
+    
+    color = NULL;
+    
+    if (!PyArg_ParseTuple (args, "s", &color))
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: wrong parameters for "
+                                     "\"get_irc_color\" function");
+        return Py_BuildValue ("i", -1);
+    }
+    
+    if (color)
+	return Py_BuildValue ("i", python_plugin->get_irc_color (python_plugin, color));
+    
+    return Py_BuildValue ("i", -1);
+}
+
+/*
+ * weechat_python_get_window_info: get infos about windows
+ */
+
+static PyObject *
+weechat_python_get_window_info (PyObject *self, PyObject *args)
+{
+    t_plugin_window_info *window_info, *ptr_window;
+    PyObject *window_list, *window_list_member;
+    
+    /* make gcc happy */
+    (void) self;
+    (void) args;
+        
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to get window info, "
+                                     "script not initialized");
+        return Py_None;
+    }
+        
+    window_list = PyList_New (0);
+    if (!window_list)
+	return Py_None;
+
+    window_info = python_plugin->get_window_info (python_plugin);
+    if  (!window_info)
+	return window_list;
+
+    for(ptr_window = window_info; ptr_window; ptr_window = ptr_window->next_window)
+    {
+	window_list_member = PyDict_New();
+	
+	if (window_list_member)
+	{
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "num_buffer"),
+			   Py_BuildValue("i", ptr_window->num_buffer));
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "win_x"),
+			   Py_BuildValue("i", ptr_window->win_x));
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "win_y"),
+			   Py_BuildValue("i", ptr_window->win_y));
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "win_width"),
+			   Py_BuildValue("i", ptr_window->win_width));
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "win_height"),
+			   Py_BuildValue("i", ptr_window->win_height));
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "win_width_pct"),
+			   Py_BuildValue("i", ptr_window->win_width_pct));
+	    PyDict_SetItem(window_list_member, Py_BuildValue("s", "win_height_pct"),
+			   Py_BuildValue("i", ptr_window->win_height_pct));
+	    
+	    PyList_Append(window_list, window_list_member);
+	}
+    }
+    
+    python_plugin->free_window_info(python_plugin, window_info);
+    
+    return window_list;
+}
+
+/*
+ * weechat_python_get_buffer_info: get infos about buffers
+ */
+
+static PyObject *
+weechat_python_get_buffer_info (PyObject *self, PyObject *args)
+{
+    t_plugin_buffer_info *buffer_info, *ptr_buffer;
+    PyObject *buffer_hash, *buffer_hash_member;
+    
+    /* make gcc happy */
+    (void) self;
+    (void) args;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to get buffer info, "
+                                     "script not initialized");
+        return Py_None;
+    }
+    
+    buffer_hash = PyDict_New ();
+    if (!buffer_hash)
+	return Py_None;
+
+    buffer_info = python_plugin->get_buffer_info (python_plugin);
+    if  (!buffer_info)
+	return buffer_hash;
+
+    for(ptr_buffer = buffer_info; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
+    {
+	buffer_hash_member = PyDict_New();
+	
+	if (buffer_hash_member)
+	{
+	    PyDict_SetItem(buffer_hash_member, Py_BuildValue("s", "type"),
+			   Py_BuildValue("i", ptr_buffer->type));
+	    PyDict_SetItem(buffer_hash_member, Py_BuildValue("s", "num_displayed"),
+			   Py_BuildValue("i", ptr_buffer->num_displayed));
+	    PyDict_SetItem(buffer_hash_member, Py_BuildValue("s", "server"),
+			   Py_BuildValue("s", ptr_buffer->server_name == NULL ? "" : ptr_buffer->server_name));
+	    PyDict_SetItem(buffer_hash_member, Py_BuildValue("s", "channel"),
+			   Py_BuildValue("s", ptr_buffer->channel_name == NULL ? "" : ptr_buffer->channel_name));
+	    PyDict_SetItem(buffer_hash_member, Py_BuildValue("s", "notify_level"),
+			   Py_BuildValue("i", ptr_buffer->notify_level));
+	    PyDict_SetItem(buffer_hash_member, Py_BuildValue("s", "log_filename"),
+			   Py_BuildValue("s", ptr_buffer->log_filename == NULL ? "" : ptr_buffer->log_filename));
+	    
+	    PyDict_SetItem(buffer_hash, 
+			   Py_BuildValue("i", ptr_buffer->number), buffer_hash_member);
+	}
+    }    
+    
+    python_plugin->free_buffer_info(python_plugin, buffer_info);
+    
+    return buffer_hash;
+}
+
+/*
+ * weechat_python_get_buffer_data: get buffer content
+ */
+
+static PyObject *
+weechat_python_get_buffer_data (PyObject *self, PyObject *args)
+{
+    t_plugin_buffer_line *buffer_data, *ptr_data;
+    PyObject *data_list, *data_list_member;
+    char *server, *channel;
+    
+    /* make gcc happy */
+    (void) self;
+    (void) args;
+    
+    if (!python_current_script)
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: unable to get buffer data, "
+                                     "script not initialized");
+        return Py_None;
+    }
+
+    if (!PyArg_ParseTuple (args, "|ss", &server, &channel))
+    {
+        python_plugin->print_server (python_plugin,
+                                     "Python error: wrong parameters for "
+                                     "\"get_buffer_data\" function");
+        return Py_None;
+    }
+
+    data_list = PyList_New (0);    
+    if (!data_list)
+	return Py_None;
+    
+    buffer_data = python_plugin->get_buffer_data (python_plugin, server, channel);
+    if (!buffer_data)
+	return data_list;
+    
+    for(ptr_data = buffer_data; ptr_data; ptr_data = ptr_data->next_line)
+    {
+	data_list_member= PyDict_New();
+	
+        if (data_list_member) 
+        {
+	    PyDict_SetItem(data_list_member, Py_BuildValue("s", "nick"),
+			   Py_BuildValue("s", ptr_data->nick == NULL ? "" : ptr_data->nick));
+	    PyDict_SetItem(data_list_member, Py_BuildValue("s", "data"),
+			   Py_BuildValue("s", ptr_data->data == NULL ? "" : ptr_data->data));
+	    	    
+            PyList_Append(data_list, data_list_member);
+        }
+    }
+    
+    python_plugin->free_buffer_data (python_plugin, buffer_data);
+    
+    return data_list;
+}
+
 /*
  * Python subroutines
  */
@@ -1256,6 +1496,7 @@ static
 PyMethodDef weechat_python_funcs[] = {
     { "register", weechat_python_register, METH_VARARGS, "" },
     { "prnt", weechat_python_print, METH_VARARGS, "" },
+    { "print_server", weechat_python_print_server, METH_VARARGS, "" },
     { "print_infobar", weechat_python_print_infobar, METH_VARARGS, "" },
     { "remove_infobar", weechat_python_remove_infobar, METH_VARARGS, "" },
     { "log", weechat_python_log, METH_VARARGS, "" },
@@ -1276,6 +1517,10 @@ PyMethodDef weechat_python_funcs[] = {
     { "get_server_info", weechat_python_get_server_info, METH_VARARGS, "" },
     { "get_channel_info", weechat_python_get_channel_info, METH_VARARGS, "" },
     { "get_nick_info", weechat_python_get_nick_info, METH_VARARGS, "" },
+    { "get_irc_color", weechat_python_get_irc_color, METH_VARARGS, "" },
+    { "get_window_info", weechat_python_get_window_info, METH_VARARGS, "" },
+    { "get_buffer_info", weechat_python_get_buffer_info, METH_VARARGS, "" },
+    { "get_buffer_data", weechat_python_get_buffer_data, METH_VARARGS, "" },
     { NULL, NULL, 0, NULL }
 };
 
@@ -1342,7 +1587,8 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
     }
 
     python_current_script = NULL;
-    
+
+    /* PyEval_AcquireLock (); */
     python_current_interpreter = Py_NewInterpreter ();
     PySys_SetArgv(1, argv);
 
@@ -1351,10 +1597,11 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
         plugin->print_server (plugin,
                               "Python error: unable to create new sub-interpreter");
         fclose (fp);
-        return 0;
+	/* PyEval_ReleaseLock (); */
+	return 0;
     }
 
-    PyThreadState_Swap (python_current_interpreter);
+    /* PyThreadState_Swap (python_current_interpreter); */
     
     weechat_module = Py_InitModule ("weechat", weechat_python_funcs);
 
@@ -1362,9 +1609,12 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
     {
         plugin->print_server (plugin,
                               "Python error: unable to initialize WeeChat module");
-        Py_EndInterpreter (python_current_interpreter);
         fclose (fp);
-        return 0;
+	
+	Py_EndInterpreter (python_current_interpreter);
+        /* PyEval_ReleaseLock (); */
+        
+	return 0;
     }
 
     /* define some constants */
@@ -1398,17 +1648,20 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
         plugin->print_server (plugin,
                               "Python error: unable to parse file \"%s\"",
                               filename);
-        free (python_current_script_filename);
+	free (python_current_script_filename);
+	fclose (fp);
+	
+	if (PyErr_Occurred ()) PyErr_Print ();
 	Py_EndInterpreter (python_current_interpreter);
-        fclose (fp);
+	/* PyEval_ReleaseLock (); */
+        
 	/* if script was registered, removing from list */
 	if (python_current_script != NULL)
 	    weechat_script_remove (plugin, &python_scripts, python_current_script);
         return 0;
     }
 
-    if (PyErr_Occurred ())
-	PyErr_Print ();
+    if (PyErr_Occurred ()) PyErr_Print ();
 
     fclose (fp);
     free (python_current_script_filename);
@@ -1417,14 +1670,19 @@ weechat_python_load (t_weechat_plugin *plugin, char *filename)
     {
         plugin->print_server (plugin,
                               "Python error: function \"register\" not found "
-                              "in file \"%s\"",
+                              "(or failed) in file \"%s\"",
                               filename);
+	
+	if (PyErr_Occurred ()) PyErr_Print ();
 	Py_EndInterpreter (python_current_interpreter);
-        return 0;
+	/* PyEval_ReleaseLock (); */
+        
+	return 0;
     }
     
     python_current_script->interpreter = (PyThreadState *) python_current_interpreter;
-        
+    /* PyEval_ReleaseThread (python_current_script->interpreter); */
+    
     return 1;
 }
 
@@ -1677,10 +1935,11 @@ weechat_plugin_init (t_weechat_plugin *plugin)
         return PLUGIN_RC_KO;
     }
 
-    PyEval_InitThreads();
-    
-    python_mainThreadState = PyThreadState_Get();
-    
+    PyEval_InitThreads();    
+    /* python_mainThreadState = PyThreadState_Swap(NULL); */
+    python_mainThreadState = PyEval_SaveThread();
+    /* PyEval_ReleaseLock (); */
+
     if (python_mainThreadState == NULL)
     {
         plugin->print_server (plugin,
@@ -1690,8 +1949,9 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     
     plugin->cmd_handler_add (plugin, "python",
                              "list/load/unload Python scripts",
-                             "[load filename] | [autoload] | [reload] | [unload]",
-                             "filename: Python script (file) to load\n\n"
+                             "[load filename] | [autoload] | [reload] | [unload [script]]",
+                             "filename: Python script (file) to load\n"
+                             "script: script name to unload\n\n"
                              "Without argument, /python command lists all loaded Python scripts.",
                              "load|autoload|reload|unload",
                              weechat_python_cmd, NULL, NULL);
@@ -1718,7 +1978,9 @@ weechat_plugin_end (t_weechat_plugin *plugin)
     /* free Python interpreter */
     if (python_mainThreadState != NULL)
     {
+	/* PyEval_AcquireLock (); */
         PyThreadState_Swap (python_mainThreadState);
+	/* PyEval_ReleaseLock (); */
         python_mainThreadState = NULL;
     }
     
