@@ -43,10 +43,10 @@ t_plugin_script *ruby_scripts = NULL;
 t_plugin_script *ruby_current_script = NULL;
 char *ruby_current_script_filename = NULL;
 
-VALUE mWeechat, mWeechatOutputs;
+VALUE ruby_mWeechat, ruby_mWeechatOutputs;
 
 #define MOD_NAME_PREFIX "WeechatRubyModule"
-int modnum = 0;
+int ruby_num = 0;
 
 typedef struct protect_call_arg {
     VALUE recv;
@@ -224,6 +224,8 @@ weechat_ruby_register (VALUE class, VALUE name, VALUE version,
     
     /* make gcc happy */
     (void) class;
+
+    ruby_current_script = NULL;
     
     if (NIL_P (name) || NIL_P (version) || NIL_P (shutdown_func) || NIL_P (description))
     {
@@ -269,13 +271,7 @@ weechat_ruby_register (VALUE class, VALUE name, VALUE version,
                                    c_name, c_version, c_description);
     }
     else
-    {
-        ruby_plugin->print_server (ruby_plugin,
-                                   "Ruby error: unable to load script "
-                                   "\"%s\" (not enough memory)",
-                                   c_name);
         return INT2FIX (0);
-    }
     
     return INT2FIX (1);
 }
@@ -340,6 +336,44 @@ weechat_ruby_print (int argc, VALUE *argv, VALUE class)
     return INT2FIX (1);
 }
 
+/*
+ * weechat_ruby_print_server: print message into a buffer server
+ */
+
+static VALUE
+weechat_ruby_print_server (VALUE class, VALUE message)
+{
+    char *c_message;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to print message, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+    
+    c_message = NULL;
+    
+    if (NIL_P (message))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"print_server\" function");
+        return INT2FIX (0);
+    }
+    
+    Check_Type (message, T_STRING);
+    c_message = STR2CSTR (message);
+
+    ruby_plugin->print_server (ruby_plugin, "%s", c_message);
+    
+    return INT2FIX (1);
+}
+
 /*
  * weechat_ruby_print_infobar: print message to infobar
  */
@@ -1028,20 +1062,8 @@ weechat_ruby_get_dcc_info (VALUE class)
 	    rb_hash_aset (dcc_list_member, rb_str_new2("cps"),
 			  INT2FIX(ptr_dcc->bytes_per_sec));
 	    
-	    if (NIL_P (rb_ary_push (dcc_list, dcc_list_member)))
-	    {
-		rb_gc_unregister_address (&dcc_list_member);
-		rb_gc_unregister_address (&dcc_list);
-		ruby_plugin->free_dcc_info (ruby_plugin, dcc_info);
-		return Qnil;
-	    }
+	    rb_ary_push (dcc_list, dcc_list_member);
 	}
-	else
-	{
-	    rb_gc_unregister_address (&dcc_list);
-	    ruby_plugin->free_dcc_info (ruby_plugin, dcc_info);
-	    return Qnil;
-	}	    
     }
     
     ruby_plugin->free_dcc_info (ruby_plugin, dcc_info);
@@ -1332,6 +1354,8 @@ weechat_ruby_get_server_info (VALUE class)
 			  INT2FIX(ptr_server->ssl_connected));
 	    rb_hash_aset (server_hash_member, rb_str_new2("nick"),
 			  rb_str_new2(ptr_server->nick));
+            rb_hash_aset (server_hash_member, rb_str_new2("nick_modes"),
+			  rb_str_new2(ptr_server->nick_modes));
 	    rb_hash_aset (server_hash_member, rb_str_new2("away_time"),
 			  rb_str_new2(timebuffer));
 	    rb_hash_aset (server_hash_member, rb_str_new2("lag"),
@@ -1488,6 +1512,231 @@ weechat_ruby_get_nick_info (VALUE class, VALUE server, VALUE channel)
     return nick_hash;
 }
 
+/*
+ * weechat_ruby_get_irc_color: 
+ *          get the numeric value which identify an irc color by its name
+ */
+
+static VALUE
+weechat_ruby_get_irc_color (VALUE class, VALUE color)
+{
+    char *c_color;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to get irc color, "
+                                   "script not initialized");
+        return INT2FIX (-1);
+    }
+    
+    c_color = NULL;
+    
+    if (NIL_P (color))
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: wrong parameters for "
+                                   "\"get_irc_color\" function");
+        return INT2FIX (-1);
+    }
+    
+    Check_Type (color, T_STRING);
+    
+    c_color = STR2CSTR (color);
+    
+    return INT2FIX (ruby_plugin->get_irc_color (ruby_plugin, c_color));
+}
+
+/*
+ * weechat_ruby_get_window_info: get infos about windows
+ */
+
+static VALUE
+weechat_ruby_get_window_info (VALUE class)
+{
+    t_plugin_window_info *window_info, *ptr_window;
+    VALUE window_list, window_list_member;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to get window info, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+
+    window_list = rb_ary_new();
+    
+    if (NIL_P (window_list))
+        return Qnil;
+    
+    window_info = ruby_plugin->get_window_info (ruby_plugin);    
+    if (!window_info)
+        return window_list;
+    
+    for(ptr_window = window_info; ptr_window; ptr_window = ptr_window->next_window)
+    {
+	window_list_member = rb_hash_new ();
+
+	if (!NIL_P (window_list_member))
+	{
+	    rb_hash_aset (window_list_member, rb_str_new2("num_buffer"), 
+			  INT2FIX(ptr_window->num_buffer));
+	    rb_hash_aset (window_list_member, rb_str_new2("win_x"),
+			  INT2FIX(ptr_window->win_x));
+	    rb_hash_aset (window_list_member, rb_str_new2("win_y"),
+			  INT2FIX(ptr_window->win_y));
+	    rb_hash_aset (window_list_member, rb_str_new2("win_width"),
+			  INT2FIX(ptr_window->win_width));
+	    rb_hash_aset (window_list_member, rb_str_new2("win_height"),
+			  INT2FIX(ptr_window->win_height));
+	    rb_hash_aset (window_list_member, rb_str_new2("win_width_pct"),
+			  INT2FIX(ptr_window->win_width_pct));
+	    rb_hash_aset (window_list_member, rb_str_new2("win_height_pct"),
+			  INT2FIX(ptr_window->win_height_pct));
+	    
+	    rb_ary_push (window_list, window_list_member);
+	}
+    }
+    
+    ruby_plugin->free_window_info (ruby_plugin, window_info);
+    
+    return window_list;
+}
+
+/*
+ * weechat_ruby_get_buffer_info: get infos about buffers
+ */
+
+static VALUE
+weechat_ruby_get_buffer_info (VALUE class)
+{
+    t_plugin_buffer_info *buffer_info, *ptr_buffer;
+    VALUE buffer_hash, buffer_hash_member;
+    
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to get buffer info, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+    
+    buffer_hash = rb_hash_new ();
+    if (!buffer_hash)
+	return Qnil;
+
+    buffer_info = ruby_plugin->get_buffer_info (ruby_plugin);
+    if  (!buffer_info)
+	return buffer_hash;
+
+    for(ptr_buffer = buffer_info; ptr_buffer; ptr_buffer = ptr_buffer->next_buffer)
+    {
+	buffer_hash_member = rb_hash_new ();
+	
+	if (buffer_hash_member)
+	{
+	    rb_hash_aset (buffer_hash_member, rb_str_new2("type"),
+			  INT2FIX(ptr_buffer->type));
+	    rb_hash_aset (buffer_hash_member, rb_str_new2("num_displayed"),
+			  INT2FIX(ptr_buffer->num_displayed));
+	    rb_hash_aset (buffer_hash_member, rb_str_new2("server"),
+			  rb_str_new2(ptr_buffer->server_name == NULL ? "" : ptr_buffer->server_name));
+	    rb_hash_aset (buffer_hash_member, rb_str_new2("channel"),
+			  rb_str_new2(ptr_buffer->channel_name == NULL ? "" : ptr_buffer->channel_name));
+	    rb_hash_aset (buffer_hash_member, rb_str_new2("notify_level"),
+			  INT2FIX(ptr_buffer->notify_level));
+	    rb_hash_aset (buffer_hash_member, rb_str_new2("log_filename"),
+			  rb_str_new2(ptr_buffer->log_filename == NULL ? "" : ptr_buffer->log_filename));
+	    
+	    rb_hash_aset (buffer_hash, INT2FIX(ptr_buffer->number), buffer_hash_member);
+	}
+    }
+    
+    ruby_plugin->free_buffer_info(ruby_plugin, buffer_info);
+    
+    return buffer_hash;
+}
+
+/*
+ * weechat_ruby_get_buffer_data: get buffer content
+ */
+
+static VALUE
+weechat_ruby_get_buffer_data (int argc, VALUE *argv, VALUE class)
+{
+    t_plugin_buffer_line *buffer_data, *ptr_data;
+    VALUE data_list, data_list_member;
+    VALUE server, channel;
+    char *c_server, *c_channel;
+
+    /* make gcc happy */
+    (void) class;
+    
+    if (!ruby_current_script)
+    {
+        ruby_plugin->print_server (ruby_plugin,
+                                   "Ruby error: unable to get buffer data, "
+                                   "script not initialized");
+        return INT2FIX (0);
+    }
+
+    server = Qnil;
+    channel = Qnil;
+    c_server = NULL;
+    c_channel = NULL;
+
+    rb_scan_args (argc, argv, "02", &server, &channel);
+    
+    if (!NIL_P (server))
+    {
+	Check_Type (server, T_STRING);
+	c_server = STR2CSTR (server);
+    }
+    
+    if (!NIL_P (channel))
+    {
+	Check_Type (channel, T_STRING);
+	c_channel = STR2CSTR (channel);
+    }
+    
+    data_list = rb_ary_new();    
+    if (NIL_P (data_list))
+        return Qnil;
+    
+    buffer_data = ruby_plugin->get_buffer_data (ruby_plugin, c_server, c_channel);
+    if (!buffer_data)
+        return data_list;
+    
+    for(ptr_data = buffer_data; ptr_data; ptr_data = ptr_data->next_line)
+    {
+	data_list_member = rb_hash_new ();
+
+	if (!NIL_P (data_list_member))
+	{
+	    rb_hash_aset (data_list_member, rb_str_new2("nick"), 
+			  rb_str_new2(ptr_data->nick == NULL ? "" : ptr_data->nick));
+	    rb_hash_aset (data_list_member, rb_str_new2("data"), 
+			  rb_str_new2(ptr_data->data == NULL ? "" : ptr_data->data));
+
+	    rb_ary_push (data_list, data_list_member);
+	}
+    }
+    
+    ruby_plugin->free_buffer_data (ruby_plugin, buffer_data);
+    
+    return data_list;
+}
+
 /*
  * weechat_ruby_output : redirection for stdout and stderr
  */
@@ -1541,8 +1790,8 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
     plugin->print_server (plugin, "Loading Ruby script \"%s\"", filename);
     ruby_current_script = NULL;
 
-    snprintf(modname, sizeof(modname), "%s%d", MOD_NAME_PREFIX, modnum);
-    modnum++;
+    snprintf(modname, sizeof(modname), "%s%d", MOD_NAME_PREFIX, ruby_num);
+    ruby_num++;
 
     curModule = rb_define_module(modname);
 
@@ -1622,7 +1871,7 @@ weechat_ruby_load (t_weechat_plugin *plugin, char *filename)
     if (ruby_current_script == NULL) {
 	plugin->print_server (plugin,
                               "Ruby error: function \"register\" not found "
-                              "in file \"%s\"",
+                              "(or failed) in file \"%s\"",
                               filename);
         return 0;
     }
@@ -1922,47 +2171,53 @@ weechat_plugin_init (t_weechat_plugin *plugin)
     ruby_init_loadpath ();
     ruby_script ("__weechat_plugin__");
     
-    mWeechat = rb_define_module("Weechat");
-    rb_define_const(mWeechat, "PLUGIN_RC_OK", INT2NUM(PLUGIN_RC_OK));
-    rb_define_const(mWeechat, "PLUGIN_RC_KO", INT2NUM(PLUGIN_RC_KO));
-    rb_define_const(mWeechat, "PLUGIN_RC_OK_IGNORE_WEECHAT", INT2NUM(PLUGIN_RC_OK_IGNORE_WEECHAT));
-    rb_define_const(mWeechat, "PLUGIN_RC_OK_IGNORE_PLUGINS", INT2NUM(PLUGIN_RC_OK_IGNORE_PLUGINS));
-    rb_define_const(mWeechat, "PLUGIN_RC_OK_IGNORE_ALL", INT2NUM(PLUGIN_RC_OK_IGNORE_ALL));    
-    rb_define_module_function (mWeechat, "register", weechat_ruby_register, 4);
-    rb_define_module_function (mWeechat, "print", weechat_ruby_print, -1);
-    rb_define_module_function (mWeechat, "print_infobar", weechat_ruby_print_infobar, 2);
-    rb_define_module_function (mWeechat, "remove_infobar", weechat_ruby_remove_infobar, -1);
-    rb_define_module_function (mWeechat, "log", weechat_ruby_log, -1);
-    rb_define_module_function (mWeechat, "command", weechat_ruby_command, -1);
-    rb_define_module_function (mWeechat, "add_message_handler", weechat_ruby_add_message_handler, 2);
-    rb_define_module_function (mWeechat, "add_command_handler", weechat_ruby_add_command_handler, -1);
-    rb_define_module_function (mWeechat, "add_timer_handler", weechat_ruby_add_timer_handler, 2);
-    rb_define_module_function (mWeechat, "add_keyboard_handler", weechat_ruby_add_keyboard_handler, 1);
-    rb_define_module_function (mWeechat, "remove_handler", weechat_ruby_remove_handler, 2);
-    rb_define_module_function (mWeechat, "remove_timer_handler", weechat_ruby_remove_timer_handler, 1);
-    rb_define_module_function (mWeechat, "remove_keyboard_handler", weechat_ruby_remove_keyboard_handler, 1);
-    rb_define_module_function (mWeechat, "get_info", weechat_ruby_get_info, -1);
-    rb_define_module_function (mWeechat, "get_dcc_info", weechat_ruby_get_dcc_info, 0);
-    rb_define_module_function (mWeechat, "get_config", weechat_ruby_get_config, 1);
-    rb_define_module_function (mWeechat, "set_config", weechat_ruby_set_config, 2);
-    rb_define_module_function (mWeechat, "get_plugin_config", weechat_ruby_get_plugin_config, 1);
-    rb_define_module_function (mWeechat, "set_plugin_config", weechat_ruby_set_plugin_config, 2);
-    rb_define_module_function (mWeechat, "get_server_info", weechat_ruby_get_server_info, 0);
-    rb_define_module_function (mWeechat, "get_channel_info", weechat_ruby_get_channel_info, 1);
-    rb_define_module_function (mWeechat, "get_nick_info", weechat_ruby_get_nick_info, 2);
-    
+    ruby_mWeechat = rb_define_module("Weechat");
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK", INT2NUM(PLUGIN_RC_OK));
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_KO", INT2NUM(PLUGIN_RC_KO));
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_WEECHAT", INT2NUM(PLUGIN_RC_OK_IGNORE_WEECHAT));
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_PLUGINS", INT2NUM(PLUGIN_RC_OK_IGNORE_PLUGINS));
+    rb_define_const(ruby_mWeechat, "PLUGIN_RC_OK_IGNORE_ALL", INT2NUM(PLUGIN_RC_OK_IGNORE_ALL));    
+    rb_define_module_function (ruby_mWeechat, "register", weechat_ruby_register, 4);
+    rb_define_module_function (ruby_mWeechat, "print", weechat_ruby_print, -1);
+    rb_define_module_function (ruby_mWeechat, "print_server", weechat_ruby_print_server, 1);
+    rb_define_module_function (ruby_mWeechat, "print_infobar", weechat_ruby_print_infobar, 2);
+    rb_define_module_function (ruby_mWeechat, "remove_infobar", weechat_ruby_remove_infobar, -1);
+    rb_define_module_function (ruby_mWeechat, "log", weechat_ruby_log, -1);
+    rb_define_module_function (ruby_mWeechat, "command", weechat_ruby_command, -1);
+    rb_define_module_function (ruby_mWeechat, "add_message_handler", weechat_ruby_add_message_handler, 2);
+    rb_define_module_function (ruby_mWeechat, "add_command_handler", weechat_ruby_add_command_handler, -1);
+    rb_define_module_function (ruby_mWeechat, "add_timer_handler", weechat_ruby_add_timer_handler, 2);
+    rb_define_module_function (ruby_mWeechat, "add_keyboard_handler", weechat_ruby_add_keyboard_handler, 1);
+    rb_define_module_function (ruby_mWeechat, "remove_handler", weechat_ruby_remove_handler, 2);
+    rb_define_module_function (ruby_mWeechat, "remove_timer_handler", weechat_ruby_remove_timer_handler, 1);
+    rb_define_module_function (ruby_mWeechat, "remove_keyboard_handler", weechat_ruby_remove_keyboard_handler, 1);
+    rb_define_module_function (ruby_mWeechat, "get_info", weechat_ruby_get_info, -1);
+    rb_define_module_function (ruby_mWeechat, "get_dcc_info", weechat_ruby_get_dcc_info, 0);
+    rb_define_module_function (ruby_mWeechat, "get_config", weechat_ruby_get_config, 1);
+    rb_define_module_function (ruby_mWeechat, "set_config", weechat_ruby_set_config, 2);
+    rb_define_module_function (ruby_mWeechat, "get_plugin_config", weechat_ruby_get_plugin_config, 1);
+    rb_define_module_function (ruby_mWeechat, "set_plugin_config", weechat_ruby_set_plugin_config, 2);
+    rb_define_module_function (ruby_mWeechat, "get_server_info", weechat_ruby_get_server_info, 0);
+    rb_define_module_function (ruby_mWeechat, "get_channel_info", weechat_ruby_get_channel_info, 1);
+    rb_define_module_function (ruby_mWeechat, "get_nick_info", weechat_ruby_get_nick_info, 2);
+    rb_define_module_function (ruby_mWeechat, "get_irc_color", weechat_ruby_get_irc_color, 1);
+    rb_define_module_function (ruby_mWeechat, "get_window_info", weechat_ruby_get_window_info, 0);
+    rb_define_module_function (ruby_mWeechat, "get_buffer_info", weechat_ruby_get_buffer_info, 0);
+    rb_define_module_function (ruby_mWeechat, "get_buffer_data", weechat_ruby_get_buffer_data, -1);
+
     /* redirect stdin and stdout */
-    mWeechatOutputs = rb_define_module("WeechatOutputs");
-    rb_define_singleton_method(mWeechatOutputs, "write", weechat_ruby_output, 1);
-    rb_define_singleton_method(mWeechatOutputs, "puts", weechat_ruby_output, 1);
-    rb_define_singleton_method(mWeechatOutputs, "p", weechat_ruby_output, 1);
-    rb_define_singleton_method(mWeechatOutputs, "flush", weechat_ruby_output_flush, 0);
+    ruby_mWeechatOutputs = rb_define_module("WeechatOutputs");
+    rb_define_singleton_method(ruby_mWeechatOutputs, "write", weechat_ruby_output, 1);
+    rb_define_singleton_method(ruby_mWeechatOutputs, "puts", weechat_ruby_output, 1);
+    rb_define_singleton_method(ruby_mWeechatOutputs, "p", weechat_ruby_output, 1);
+    rb_define_singleton_method(ruby_mWeechatOutputs, "flush", weechat_ruby_output_flush, 0);
     
 
     plugin->cmd_handler_add (plugin, "ruby",
                              "list/load/unload Ruby scripts",
-                             "[load filename] | [autoload] | [reload] | [unload]",
-                             "filename: Ruby script (file) to load\n\n"
+                             "[load filename] | [autoload] | [reload] | [unload [script]]",
+                             "filename: Ruby script (file) to load\n"
+                             "script: script name to unload\n\n"
                              "Without argument, /ruby command lists all loaded Ruby scripts.",
                              "load|autoload|reload|unload",
                              weechat_ruby_cmd, NULL, NULL);
